#!/usr/bin/env python

# This example reads a volume dataset, extracts an isosurface that
# represents the skin and displays it.

import vtk
from vtk.util.misc import vtkGetDataRoot
VTK_DATA_ROOT = vtkGetDataRoot()

# Create the renderer, the render window, and the interactor. The
# renderer draws into the render window, the interactor enables mouse-
# and keyboard-based interaction with the scene.
aRenderer = vtk.vtkRenderer()
renWin = vtk.vtkRenderWindow()
renWin.AddRenderer(aRenderer)
iren = vtk.vtkRenderWindowInteractor()
iren.SetRenderWindow(renWin)

# The following reader is used to read a series of 2D slices (images)
# that compose the volume. The slice dimensions are set, and the
# pixel spacing. The data Endianness must also be specified. The reader
# usese the FilePrefix in combination with the slice number to construct
# filenames using the format FilePrefix.%d. (In this case the FilePrefix
# is the root name of the file: quarter.)
v16 = vtk.vtkVolume16Reader()
v16.SetDataDimensions(64, 64)
v16.SetDataByteOrderToLittleEndian()
v16.SetFilePrefix(VTK_DATA_ROOT + "/Data/headsq/quarter")
v16.SetImageRange(1, 93)
v16.SetDataSpacing(3.2, 3.2, 1.5)

# An isosurface, or contour value of 500 is known to correspond to the
# skin of the patient. Once generated, a vtkPolyDataNormals filter is
# is used to create normals for smooth surface shading during rendering.
# The triangle stripper is used to create triangle strips from the
# isosurface these render much faster on may systems.
skinExtractor = vtk.vtkContourFilter()
skinExtractor.SetInputConnection(v16.GetOutputPort())
skinExtractor.SetValue(0, 500)
skinNormals = vtk.vtkPolyDataNormals()
skinNormals.SetInputConnection(skinExtractor.GetOutputPort())
skinNormals.SetFeatureAngle(60.0)
skinMapper = vtk.vtkPolyDataMapper()
skinMapper.SetInputConnection(skinNormals.GetOutputPort())
skinMapper.ScalarVisibilityOff()
skin = vtk.vtkActor()
skin.SetMapper(skinMapper)

# An outline provides context around the data.
outlineData = vtk.vtkOutlineFilter()
outlineData.SetInputConnection(v16.GetOutputPort())
mapOutline = vtk.vtkPolyDataMapper()
mapOutline.SetInputConnection(outlineData.GetOutputPort())
outline = vtk.vtkActor()
outline.SetMapper(mapOutline)
outline.GetProperty().SetColor(0, 0, 0)

# It is convenient to create an initial view of the data. The FocalPoint
# and Position form a vector direction. Later on (ResetCamera() method)
# this vector is used to position the camera to look at the data in
# this direction.
aCamera = vtk.vtkCamera()
aCamera.SetViewUp(0, 0, -1)
aCamera.SetPosition(0, 1, 0)
aCamera.SetFocalPoint(0, 0, 0)
aCamera.ComputeViewPlaneNormal()

# Actors are added to the renderer. An initial camera view is created.
# The Dolly() method moves the camera towards the FocalPoint,
# thereby enlarging the image.
aRenderer.AddActor(outline)
aRenderer.AddActor(skin)
aRenderer.SetActiveCamera(aCamera)
aRenderer.ResetCamera()
aCamera.Dolly(1.5)

# Set a background color for the renderer and set the size of the
# render window (expressed in pixels).
aRenderer.SetBackground(1, 1, 1)
renWin.SetSize(640, 480)

# Note that when camera movement occurs (as it does in the Dolly()
# method), the clipping planes often need adjusting. Clipping planes
# consist of two planes: near and far along the view direction. The
# near plane clips out objects in front of the plane the far plane
# clips out objects behind the plane. This way only what is drawn
# between the planes is actually rendered.
aRenderer.ResetCameraClippingRange()

# Interact with the data.
iren.Initialize()
renWin.Render()
iren.Start()



###############################################################################
##
## Copyright (C) 2014-2015, New York University.
## Copyright (C) 2011-2014, NYU-Poly.
## Copyright (C) 2006-2011, University of Utah.
## All rights reserved.
## Contact: contact@vistrails.org
##
## This file is part of VisTrails.
##
## "Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions are met:
##
##  - Redistributions of source code must retain the above copyright notice,
##    this list of conditions and the following disclaimer.
##  - Redistributions in binary form must reproduce the above copyright
##    notice, this list of conditions and the following disclaimer in the
##    documentation and/or other materials provided with the distribution.
##  - Neither the name of the New York University nor the names of its
##    contributors may be used to endorse or promote products derived from
##    this software without specific prior written permission.
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
##
###############################################################################
from __future__ import division

from vistrails.core.bundles.pyimport import py_import
import vistrails.core.requirements
from vistrails.gui.modules.source_configure import SourceConfigurationWidget
from PyQt4 import QtCore, QtGui
from vistrails.gui.theme import CurrentTheme

def TextEditor(parent=None):
    try:
        py_import('PyQt4.Qsci', {'linux-ubuntu': 'python-qscintilla2'})
    except ImportError:
        return OldTextEditor(parent)
    else:
        return NewTextEditor(parent)

def NewTextEditor(parent):
    vistrails.core.requirements.require_python_module('PyQt4.Qsci')
    from PyQt4.Qsci import QsciScintilla
    class _TextEditor(QsciScintilla):
    
        def __init__(self, parent=None):
            QsciScintilla.__init__(self, parent)
            ## set the default font of the editor
            ## and take the same font for line numbers
            font = CurrentTheme.PYTHON_SOURCE_EDITOR_FONT
            self.setFont(font)
            fm = QtGui.QFontMetrics(font)
        
            ## Line numbers
            # conventionally, margin 0 is for line numbers
            self.setMarginWidth(0, fm.width( "0000" ) + 4)
            self.setMarginLineNumbers(0, True)

            self.setAutoIndent(True)

            ## Edge Mode shows a red vetical bar at 80 chars
            self.setEdgeMode(QsciScintilla.EdgeLine)
            self.setEdgeColumn(80)
            self.setEdgeColor(QtGui.QColor("#CCCCCC"))
        
            ## Folding visual : we will use boxes
            self.setFolding(QsciScintilla.BoxedTreeFoldStyle)
        
            ## Braces matching
            self.setBraceMatching(QsciScintilla.SloppyBraceMatch)
        
            ## Editing line color
        #    self.setCaretLineVisible(True)
        #    self.setCaretLineBackgroundColor(QtGui.QColor("#CDA869"))
            
            
                    ## Margins colors
            # line numbers margin
            self.setMarginsBackgroundColor(QtGui.QColor("#FFFFFF"))
            self.setMarginsForegroundColor(QtGui.QColor("#000000"))
        
            # folding margin colors (foreground,background)
            self.setFoldMarginColors(QtGui.QColor("#DDDDDD"),QtGui.QColor("#DDDDDD"))
            # do not use tabs
            self.setIndentationsUseTabs(False)
            self.setTabWidth(4)
            self.setTabIndents(True)
        
            # set autocompletion
            self.setAutoCompletionThreshold(2)
            self.setAutoCompletionSource(QsciScintilla.AcsDocument)
            self.setAutoCompletionCaseSensitivity(True)
            self.setAutoCompletionReplaceWord(True)
            self.setAutoCompletionFillupsEnabled(True)
            
        def setPlainText(self, text):
            """ setPlainText(text: str) -> None
            redirect to setText
            
            """
            self.setText(text)
        
        def toPlainText(self):
            """ setPlainText(text: str) -> None
            redirect to self.text()
            
            """
            text = self.text()
            return text.replace('\r\n', '\n').replace('\r', '\n')
#        def focusOutEvent(self, event):
#            if self.parent():
#                QtCore.QCoreApplication.sendEvent(self.parent(), event)
#            QsciScintilla.focusOutEvent(self, event)

    return _TextEditor(parent)

class OldTextEditor(QtGui.QTextEdit):

    def __init__(self, parent=None):
        QtGui.QTextEdit.__init__(self, parent)
        self.setLineWrapMode(QtGui.QTextEdit.NoWrap)
        self.formatChanged(None)
        self.setCursorWidth(8)
        self.connect(self,
                     QtCore.SIGNAL('currentCharFormatChanged(QTextCharFormat)'),
                     self.formatChanged)

    def formatChanged(self, f):
        self.setFont(CurrentTheme.PYTHON_SOURCE_EDITOR_FONT)

    def keyPressEvent(self, event):
        """ keyPressEvent(event: QKeyEvent) -> Nont
        Handle tab with 4 spaces
        
        """
        if event.key()==QtCore.Qt.Key_Tab:
            self.insertPlainText('    ')
        else:
            # super(PythonEditor, self).keyPressEvent(event)
            QtGui.QTextEdit.keyPressEvent(self, event)
            
class TextConfigurationWidget(SourceConfigurationWidget):
    def __init__(self, module, controller, parent=None):
        SourceConfigurationWidget.__init__(self, module, controller, 
                                           TextEditor, False, False, parent, False, portName='value')



###############################################################################
##
## Copyright (C) 2014-2015, New York University.
## Copyright (C) 2011-2014, NYU-Poly.
## Copyright (C) 2006-2011, University of Utah.
## All rights reserved.
## Contact: contact@vistrails.org
##
## This file is part of VisTrails.
##
## "Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions are met:
##
##  - Redistributions of source code must retain the above copyright notice,
##    this list of conditions and the following disclaimer.
##  - Redistributions in binary form must reproduce the above copyright
##    notice, this list of conditions and the following disclaimer in the
##    documentation and/or other materials provided with the distribution.
##  - Neither the name of the New York University nor the names of its
##    contributors may be used to endorse or promote products derived from
##    this software without specific prior written permission.
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
##
###############################################################################
from __future__ import division

from PyQt4 import QtCore, QtGui
from vistrails.core.utils import VistrailsInternalError
from vistrails.core.vistrail.port import PortEndPoint
from vistrails.gui.utils import show_question, SAVE_BUTTON, DISCARD_BUTTON
from vistrails.gui.common_widgets import QPromptWidget

class StandardModuleConfigurationWidget(QtGui.QWidget):

    def __init__(self, module, controller, parent=None):
        QtGui.QWidget.__init__(self, parent)
        self.module = module
        self.controller = controller
        self.state_changed = False
        
    def activate(self):
        #reimplement this to set the focus to a certain widget when activated
        pass
    
    def askToSaveChanges(self):
        if self.state_changed:
            message = ('Configuration panel contains unsaved changes. '
                      'Do you want to save changes before proceeding?' )
            res = show_question('VisTrails',
                                message,
                                buttons = [SAVE_BUTTON, DISCARD_BUTTON])
            if res == SAVE_BUTTON:
                self.saveTriggered()
                return True
            else:
                self.resetTriggered()
                return False
            
    def saveTriggered(self):
        msg = "Must implement saveTriggered in subclass"
        raise VistrailsInternalError(msg)
    
    def resetTriggered(self):
        msg = "Must implement saveTriggered in subclass"
        raise VistrailsInternalError(msg)

class DefaultModuleConfigurationWidget(StandardModuleConfigurationWidget):
    def __init__(self, module, controller, parent=None):
        StandardModuleConfigurationWidget.__init__(self, module, controller, 
                                                   parent)
        self.prompt = QPromptWidget()
        self.prompt.setPromptText("Please use the visibility icon (an eye) in" 
                                  " the Module Information panel to show or"
                                  " hide a port.")
        layout = QtGui.QVBoxLayout()
        layout.addWidget(self.prompt)
        self.setLayout(layout)
        self.prompt.showPrompt()
        
    def saveTriggered(self):
        pass
    
    def resetTriggered(self):
        pass
    
class _DefaultModuleConfigurationWidget(StandardModuleConfigurationWidget):
    """ This is the Default ModuleConfigurationWidget that shows a list of
        ports to be enabled or disabled """
    def __init__(self, module, controller, parent=None):
        StandardModuleConfigurationWidget.__init__(self, module, controller, 
                                                   parent)
       
        self.setLayout(QtGui.QVBoxLayout())
        self.layout().setMargin(0)
        self.layout().setSpacing(0)
        self.scrollArea = QtGui.QScrollArea(self)
        self.layout().addWidget(self.scrollArea)
        self.scrollArea.setFrameStyle(QtGui.QFrame.NoFrame)
        self.listContainer = QtGui.QWidget(self.scrollArea)
        self.listContainer.setLayout(QtGui.QGridLayout(self.listContainer))
        self.listContainer.setFocusPolicy(QtCore.Qt.WheelFocus)
        self.inputPorts = self.module.destinationPorts()
        self.inputDict = {}
        self.outputPorts = self.module.sourcePorts()
        self.outputDict = {}
        self.constructList()
        self.buttonLayout = QtGui.QHBoxLayout()
        self.buttonLayout.setMargin(5)
        self.saveButton = QtGui.QPushButton('&Save', self)
        self.saveButton.setFixedWidth(100)
        self.saveButton.setEnabled(False)
        self.buttonLayout.addWidget(self.saveButton)
        self.resetButton = QtGui.QPushButton('&Reset', self)
        self.resetButton.setFixedWidth(100)
        self.resetButton.setEnabled(False)
        self.buttonLayout.addWidget(self.resetButton)
        
        self.layout().addLayout(self.buttonLayout)
        self.connect(self.saveButton, QtCore.SIGNAL('clicked(bool)'), 
                     self.saveTriggered)
        self.connect(self.resetButton, QtCore.SIGNAL('clicked(bool)'), 
                     self.resetTriggered)
        self.setMouseTracking(True)
        self.setFocusPolicy(QtCore.Qt.WheelFocus)
        self.adjustSize()
#        self.mouseOver = False
        self.state_changed = False
        
    ###########################################################################
    # Focus event handlers
    # There is something wrong with focus on this widget, because I couldn't
    # get it to work with just self.setFocusPolicy() and reimplementing 
    # focusOutEvent. No focus events were being generated at all
    # I had to force calling setFocus(reason) when something changed
    # and as this was also generating focusOutEvents when it shouldn't. 
    # So I also had to force to emit focusOutEvent only when the mouse is not 
    # over this panel.
    # I already spent a lot of time trying fix and I couldn't.
    # If anybody has the time to fix this, please do it. (Emanuele)     
    #
    # DAK -- seems to be ok when in the palette window...

#    def focusOutEvent(self, event):
#        #self.askToSaveChanges()
#        QtGui.QWidget.focusOutEvent(self, event)
#        
#    def enterEvent(self, event):
#        self.mouseOver = True
#        
#    def leaveEvent(self, event):
#        self.mouseOver = False
        
    def checkBoxFromPort(self, port, input_=False):
        checkBox = QtGui.QCheckBox(port.name)
        if input_:
            port_visible = port.name in self.module.visible_input_ports
        else:
            port_visible = port.name in self.module.visible_output_ports
        if not port.optional or port_visible:
            checkBox.setCheckState(QtCore.Qt.Checked)
        else:
            checkBox.setCheckState(QtCore.Qt.Unchecked)
        if not port.optional or (input_ and port.sigstring=='()'):
            checkBox.setEnabled(False)
        return checkBox

    def constructList(self):
        label = QtGui.QLabel('Input Ports')
        label.setAlignment(QtCore.Qt.AlignHCenter)
        label.font().setBold(True)
        label.font().setPointSize(12)
        self.listContainer.layout().addWidget(label, 0, 0)
        label = QtGui.QLabel('Output Ports')
        label.setAlignment(QtCore.Qt.AlignHCenter)
        label.font().setBold(True)
        label.font().setPointSize(12)
        self.listContainer.layout().addWidget(label, 0, 1)

        for i in xrange(len(self.inputPorts)):
            port = self.inputPorts[i]
            checkBox = self.checkBoxFromPort(port, True)
            checkBox.setFocusPolicy(QtCore.Qt.StrongFocus)
            self.connect(checkBox, QtCore.SIGNAL("stateChanged(int)"),
                         self.updateState)
            self.inputDict[port.name] = checkBox
            self.listContainer.layout().addWidget(checkBox, i+1, 0)
        
        for i in xrange(len(self.outputPorts)):
            port = self.outputPorts[i]
            checkBox = self.checkBoxFromPort(port)
            checkBox.setFocusPolicy(QtCore.Qt.StrongFocus)
            self.connect(checkBox, QtCore.SIGNAL("stateChanged(int)"),
                         self.updateState)
            self.outputDict[port.name] = checkBox
            self.listContainer.layout().addWidget(checkBox, i+1, 1)
        
        self.listContainer.adjustSize()
        self.listContainer.setFixedHeight(self.listContainer.height())
        self.scrollArea.setWidget(self.listContainer)
        self.scrollArea.setWidgetResizable(True)

    def sizeHint(self):
        return QtCore.QSize(384, 512)
        
    def saveTriggered(self, checked = False):
        for port in self.inputPorts:
            if (port.optional and
                self.inputDict[port.name].checkState()==QtCore.Qt.Checked):
                self.module.visible_input_ports.add(port.name)
            else:
                self.module.visible_input_ports.discard(port.name)
            
        for port in self.outputPorts:
            if (port.optional and
                self.outputDict[port.name].checkState()==QtCore.Qt.Checked):
                self.module.visible_output_ports.add(port.name)
            else:
                self.module.visible_output_ports.discard(port.name)
        self.saveButton.setEnabled(False)
        self.resetButton.setEnabled(False)
        self.state_changed = False
        self.emit(QtCore.SIGNAL("stateChanged"))
        self.emit(QtCore.SIGNAL('doneConfigure'), self.module.id)
        
    def closeEvent(self, event):
        self.askToSaveChanges()
        event.accept()
                
    def resetTriggered(self):
        self.setFocus(QtCore.Qt.MouseFocusReason)
        self.setUpdatesEnabled(False)
        for i in xrange(len(self.inputPorts)):
            port = self.inputPorts[i]
            entry = (PortEndPoint.Destination, port.name)
            checkBox = self.inputDict[port.name]
            if not port.optional or entry in self.module.portVisible:
                checkBox.setCheckState(QtCore.Qt.Checked)
            else:
                checkBox.setCheckState(QtCore.Qt.Unchecked)
            if not port.optional or port.sigstring=='()':
                checkBox.setEnabled(False)
        for i in xrange(len(self.outputPorts)):
            port = self.outputPorts[i]
            entry = (PortEndPoint.Source, port.name)
            checkBox = self.outputDict[port.name]
            if not port.optional or entry in self.module.portVisible:
                checkBox.setCheckState(QtCore.Qt.Checked)
            else:
                checkBox.setCheckState(QtCore.Qt.Unchecked)
            if not port.optional:
                checkBox.setEnabled(False)
        self.setUpdatesEnabled(True)
        self.saveButton.setEnabled(False)
        self.resetButton.setEnabled(False)
        self.state_changed = False
        self.emit(QtCore.SIGNAL("stateChanged"))
        
    def updateState(self, state):
        self.setFocus(QtCore.Qt.MouseFocusReason)
        self.saveButton.setEnabled(True)
        self.resetButton.setEnabled(True)
        if not self.state_changed:
            self.state_changed = True
            self.emit(QtCore.SIGNAL("stateChanged"))



###############################################################################
##
## Copyright (C) 2014-2015, New York University.
## Copyright (C) 2011-2014, NYU-Poly.
## Copyright (C) 2006-2011, University of Utah.
## All rights reserved.
## Contact: contact@vistrails.org
##
## This file is part of VisTrails.
##
## "Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions are met:
##
##  - Redistributions of source code must retain the above copyright notice,
##    this list of conditions and the following disclaimer.
##  - Redistributions in binary form must reproduce the above copyright
##    notice, this list of conditions and the following disclaimer in the
##    documentation and/or other materials provided with the distribution.
##  - Neither the name of the New York University nor the names of its
##    contributors may be used to endorse or promote products derived from
##    this software without specific prior written permission.
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
##
###############################################################################
from __future__ import division

import copy
from itertools import izip

from vistrails.core.vistrail.annotation import Annotation
from vistrails.core.vistrail.location import Location
from vistrails.core.vistrail.module import Module
from vistrails.core.vistrail.module_control_param import ModuleControlParam
from vistrails.core.vistrail.module_function import ModuleFunction
from vistrails.core.vistrail.port_spec import PortSpec, PortEndPoint
from vistrails.db.domain import DBGroup

from vistrails.core.utils import NoSummon, VistrailsInternalError, report_stack
from vistrails.core.modules.basic_modules import identifier as basic_pkg, \
    version as basic_pkg_version
import vistrails.core.modules.sub_module
import vistrails.core.modules.module_registry
from vistrails.core.modules.module_registry import get_module_registry, ModuleRegistry

import unittest
from vistrails.db.domain import IdScope

class Group(DBGroup, Module):

    ##########################################################################
    # Constructors and copy

    def __init__(self, *args, **kwargs):
        if 'pipeline' in kwargs:
            kwargs['workflow'] = kwargs['pipeline']
            del kwargs['pipeline']
        DBGroup.__init__(self, *args, **kwargs)
        if self.cache is None:
            self.cache = 1
        if self.id is None:
            self.id = -1
        if self.location is None:
            self.location = Location(x=-1.0, y=-1.0)
        if self.name is None:
            self.name = ''
        if self.package is None:
            self.package = ''
        if self.version is None:
            self.version = ''
        self.set_defaults()

    def set_defaults(self, other=None):
        Module.set_defaults(self, other)

    def setup_indices(self):        
        # delay this until needed (on materialize)
        self._port_specs = None
        self._port_specs_id_index = None
        # self.make_port_specs()

    def __copy__(self):
        return Group.do_copy(self)

    def do_copy(self, new_ids=False, id_scope=None, id_remap=None):
        cp = DBGroup.do_copy(self, new_ids, id_scope, id_remap)
        cp.__class__ = Group
        cp.set_defaults(self)
        return cp

    @staticmethod
    def convert(_group):
        if _group.__class__ == Group:
            return
        _group.__class__ = Group
        if _group.db_location:
            Location.convert(_group.db_location)
        if _group.db_workflow:
            from vistrails.core.vistrail.pipeline import Pipeline
            Pipeline.convert(_group.db_workflow)
        for _function in _group.db_functions:
            ModuleFunction.convert(_function)
        for _annotation in _group.db_get_annotations():
            Annotation.convert(_annotation)
        for _control_parameter in _group.db_get_controlParameters():
            ModuleControlParam.convert(_control_parameter)
        _group.set_defaults()

    ##########################################################################
    # Properties

    # We need to repeat these here because Module uses DBModule. ...
    id = DBGroup.db_id
    cache = DBGroup.db_cache
    annotations = DBGroup.db_annotations
    control_parameters = DBGroup.db_controlParameters
    location = DBGroup.db_location
    center = DBGroup.db_location
    # version = DBGroup.db_version
    # name = DBGroup.db_name
    # label = DBGroup.db_name
    # namespace = DBGroup.db_namespace
    # package = DBGroup.db_package

    name = 'Group'
    label = 'Group'
    package = basic_pkg
    namespace = None
    version = basic_pkg_version
    internal_version = ''

    def is_group(self):
        return True

    def _get_pipeline(self):
        return self.db_workflow
    def _set_pipeline(self, pipeline):
        self.db_workflow = pipeline
        self.setup_indices()
    pipeline = property(_get_pipeline, _set_pipeline)
    workflow = pipeline
    
    # override these from the Module class with defaults
    # these are "local" port_specs, but Group's are "registry"
    def _get_port_specs(self):
        if self._port_specs_id_index is None:
            if self._port_specs is None:
                self.make_port_specs()
            self._port_specs_id_index = {}
            self._port_specs_id_index = \
                dict([(p.id, p) for p in self._port_specs.itervalues()])
        return self._port_specs_id_index
    port_specs = property(_get_port_specs)

    def _get_port_spec_list(self):
        if self._port_specs is None:
            self.make_port_specs()
        return self._port_specs.values()
    port_spec_list = property(_get_port_spec_list)

    def has_portSpec_with_name(self, name):
        if self._port_specs is None:
            self.make_port_specs()
        return name in self._port_specs

    def get_portSpec_by_name(self, name):
        if self._port_specs is None:
            self.make_port_specs()
        if name in self._port_specs:
            return self._port_specs[name]
        return None

    def add_port_spec(self, spec):
        # operate on self._port_specs instead of db level
        if self._port_specs is None:
            self.make_port_specs()
        self._port_specs[(spec.name, spec.type)] = spec
        self.port_specs[spec.id] = spec
        if spec.type == 'input':
            self._input_port_specs.append(spec)
        elif spec.type == 'output':
            self._output_port_specs.append(spec)
    def delete_port_spec(self, spec):
        if spec.type == 'input':
            self._input_port_specs.remove(spec)
        elif spec.type == 'output':
            self._output_port_specs.remove(spec)
        # operate on self._port_specs instead of db level
        del self._port_specs[(spec.name, spec.type)]
        del self.port_specs[spec.id]

    def _get_input_port_specs(self):
        if self._port_specs is None:
            self.make_port_specs()
        return Module._get_input_port_specs(self)
    input_port_specs = property(_get_input_port_specs)
    def _get_output_port_specs(self):
        if self._port_specs is None:
            self.make_port_specs()
        return Module._get_output_port_specs(self)
    output_port_specs = property(_get_output_port_specs)

    def get_port_spec_info(self, module):
        return vistrails.core.modules.sub_module.get_port_spec_info(self.pipeline, 
                                                          module)

    def make_port_specs(self):
        self._port_specs = {}
        self._input_port_specs = []
        self._output_port_specs = []
        self._input_remap = {}
        self._output_remap = {}
        if self.pipeline is None:
            return

        registry = get_module_registry()
        for module in self.pipeline.module_list:
            if module.name == 'OutputPort' and module.package == basic_pkg:
                (port_name, sigstring, optional, depth, _) = \
                    self.get_port_spec_info(module)
                port_spec = registry.create_port_spec(port_name, 'output',
                                                      None, sigstring,
                                                      optional, depth=depth)
                self._port_specs[(port_name, 'output')] = port_spec
                self._output_port_specs.append(port_spec)
                self._output_remap[port_name] = module
            elif module.name == 'InputPort' and module.package == basic_pkg:
                (port_name, sigstring, optional, depth, _) = \
                    self.get_port_spec_info(module)
                port_spec = registry.create_port_spec(port_name, 'input',
                                                      None, sigstring,
                                                      optional, depth=depth)
                self._port_specs[(port_name, 'input')] = port_spec
                self._input_port_specs.append(port_spec)
                self._input_remap[port_name] = module

    ##########################################################################
    # Operators
    
    def __str__(self):
        """__str__() -> str - Returns a string representation of an 
        GroupModule object. 

        """
        rep = '<group id="%s">'
        rep += str(self.location)
        rep += str(self.functions)
        rep += str(self.annotations)
        rep += '</group>'
        return  rep % str(self.id)

    def __eq__(self, other):
        """ __eq__(other: GroupModule) -> boolean
        Returns True if self and other have the same attributes. Used by == 
        operator. 
        
        """
        if type(other) != type(self):
            return False
        if self.location != other.location:
            return False
        if len(self.functions) != len(other.functions):
            return False
        if len(self.annotations) != len(other.annotations):
            return False
        for f,g in izip(self.functions, other.functions):
            if f != g:
                return False
        for f,g in izip(self.annotations, other.annotations):
            if f != g:
                return False
        return True

    def __ne__(self, other):
        return not self.__eq__(other)


################################################################################
# Testing


class TestGroup(unittest.TestCase):

    def create_group(self, id_scope=IdScope()):
        from vistrails.core.vistrail.location import Location
        from vistrails.core.vistrail.module_function import ModuleFunction
        from vistrails.core.vistrail.module_param import ModuleParam

        params = [ModuleParam(id=id_scope.getNewId(ModuleParam.vtType),
                                  type='Int',
                                  val='1')]
        functions = [ModuleFunction(id=id_scope.getNewId(ModuleFunction.vtType),
                                    name='value',
                                    parameters=params)]
        location = Location(id=id_scope.getNewId(Location.vtType),
                            x=12.342,
                            y=-19.432)
        module = \
            Group(id=id_scope.getNewId(Group.vtType),
                  location=location,
                  functions=functions,
                  )
        return module

    def test_copy(self):
        """Check that copy works correctly"""
        
        id_scope = IdScope()
        m1 = self.create_group(id_scope)
        m2 = copy.copy(m1)
        self.assertEquals(m1, m2)
        self.assertEquals(m1.id, m2.id)
        m3 = m1.do_copy(True, id_scope, {})
        self.assertEquals(m1, m3)
        self.assertNotEquals(m1.id, m3.id)

    def test_serialization(self):
        """ Check that serialize and unserialize are working properly """
        import vistrails.core.db.io

        m1 = self.create_group()
        xml_str = vistrails.core.db.io.serialize(m1)
        m2 = vistrails.core.db.io.unserialize(xml_str, Group)
        self.assertEquals(m1, m2)
        self.assertEquals(m1.id, m2.id)



###############################################################################
##
## Copyright (C) 2014-2015, New York University.
## Copyright (C) 2011-2014, NYU-Poly.
## Copyright (C) 2006-2011, University of Utah.
## All rights reserved.
## Contact: contact@vistrails.org
##
## This file is part of VisTrails.
##
## "Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions are met:
##
##  - Redistributions of source code must retain the above copyright notice,
##    this list of conditions and the following disclaimer.
##  - Redistributions in binary form must reproduce the above copyright
##    notice, this list of conditions and the following disclaimer in the
##    documentation and/or other materials provided with the distribution.
##  - Neither the name of the New York University nor the names of its
##    contributors may be used to endorse or promote products derived from
##    this software without specific prior written permission.
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
##
###############################################################################
from __future__ import division

import vistrails.core.modules.module_registry
from vistrails.core.modules.vistrails_module import Module, ModuleError
from ast import literal_eval
from PyQt4 import QtCore, QtGui
import vistrails.api

class QPythonCalc(QtGui.QWidget):
    """QPythonCalc is a widget used for specifying PythonCalc
    parameters and providing two buttons for compute and add that
    module to the pipeline. """
    
    def __init__(self, parent=None):
        """ init(parent: QWidget) -> None        
        We construct a simple GUI with edit boxes for 2 operands and a
        combo box for the operator. There are also 2 buttons: (1)
        perform the calculation and display the result, (2) create a
        PythonCalc module with the specified parameters and add to the
        current pipeline.

        """
        QtGui.QWidget.__init__(self, parent)
        self.setWindowTitle('PythonCalc GUI')
        gridLayout = QtGui.QGridLayout()
        self.setLayout(gridLayout)
        gridLayout.addWidget(QtGui.QLabel('value1'), 0, 0)
        gridLayout.addWidget(QtGui.QLabel('value2'), 1, 0)
        gridLayout.addWidget(QtGui.QLabel('op'), 2, 0)
        gridLayout.addWidget(QtGui.QLabel('result'), 3, 0)
        
        numberValidator = QtGui.QIntValidator(self)
        self.value1Edit = QtGui.QLineEdit('0')
        self.value2Edit = QtGui.QLineEdit('0')
        self.value1Edit.setValidator(numberValidator)
        self.value2Edit.setValidator(numberValidator)
        gridLayout.addWidget(self.value1Edit, 0, 1)
        gridLayout.addWidget(self.value2Edit, 1, 1)
        
        self.opCombo = QtGui.QComboBox()
        self.opCombo.addItems(['+', '-', '*', '/'])
        gridLayout.addWidget(self.opCombo, 2, 1)

        self.resultLabel = QtGui.QLabel()
        gridLayout.addWidget(self.resultLabel, 3, 1)

        self.calculateButton = QtGui.QPushButton('Calculate')
        gridLayout.addWidget(self.calculateButton, 4, 0)
        self.connect(self.calculateButton, QtCore.SIGNAL('clicked()'),
                     self.calculate)

        self.createModuleButton = QtGui.QPushButton('Create Module')
        gridLayout.addWidget(self.createModuleButton, 4, 1)
        self.connect(self.createModuleButton, QtCore.SIGNAL('clicked()'),
                     self.createModule)

    def calculate(self):
        """ calculate() -> None        
        Perform the calculation based on value1, value2 and op,
        similar to the PythonCalc module, but through the GUI.

        """
        result = literal_eval(self.value1Edit.text() +
                              self.opCombo.currentText() +
                              self.value2Edit.text())
        self.resultLabel.setText(str(result))

    def createModule(self):
        """ createModule() -> None
        Collect parameters, values and operator, and create a
        PythonCalc module to the current pipeline using the API
        provided in vistrails.api.

        """
        pythoncalc = "org.vistrails.vistrails.pythoncalc"
        module = vistrails.api.add_module(0, 0, pythoncalc, 'PythonCalc', '')
        vistrails.api.get_current_controller().update_function(module, 'value1',
                                                     [str(self.value1Edit.text())])
        vistrails.api.get_current_controller().update_function(module, 'value2',
                                                     [str(self.value2Edit.text())])
        vistrails.api.get_current_controller().update_function(module, 'op',
                                                     [str(self.opCombo.currentText())])
        vistrails.api.switch_to_pipeline_view()

def initialize(*args, **keywords):
    """ initialize() -> None    
    Package-entry to initialize the package. We just create and show
    the main window, a QPythonCalc widget, here.
    
    """
    global mainWindow
    mainWindow = QPythonCalc()
    mainWindow.show()

def menu_items():
    """menu_items() -> tuple of (str,function)
    It returns a list of pairs containing text for the menu and a
    callback function that will be executed when that menu item is
    selected. This will add a 'Create Module' menu item under the
    Packages>PythonCalcQt menu. It has the same functionality as the
    'Create Module' button, which adds a PythonCalc with specified
    parameters module to the current pipeline.
    
    """
    lst = []
    global mainWindow
    lst.append(("Create Module", mainWindow.createModule))
    return tuple(lst)    



#!/usr/bin/env python
###############################################################################
##
## Copyright (C) 2014-2015, New York University.
## Copyright (C) 2011-2014, NYU-Poly.
## Copyright (C) 2006-2011, University of Utah.
## All rights reserved.
## Contact: contact@vistrails.org
##
## This file is part of VisTrails.
##
## "Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions are met:
##
##  - Redistributions of source code must retain the above copyright notice,
##    this list of conditions and the following disclaimer.
##  - Redistributions in binary form must reproduce the above copyright
##    notice, this list of conditions and the following disclaimer in the
##    documentation and/or other materials provided with the distribution.
##  - Neither the name of the New York University nor the names of its
##    contributors may be used to endorse or promote products derived from
##    this software without specific prior written permission.
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
##
###############################################################################

import os
import sys
import tempfile
sys.path.append(os.path.join(os.path.dirname(os.path.dirname(
                os.path.abspath(__file__))), 'vistrails'))
from vistrails.db.services import io
from vistrails.db.versions import currentVersion
from db_utils import parse_db_cmd_line

# DO NOT RUN THIS WHILE USERS HAVE ACCESS TO THE DATABASE!
def update_db(config, new_version=None, tmp_dir=None, restore=False):
    obj_types = {'vistrail': io.open_bundle_from_db,
                 # 'workflow': io.open_from_db, 
                 # 'log': io.open_from_db,
                 # 'registry': io.open_from_db
                 }
    if new_version is None:
        new_version = currentVersion
    if tmp_dir is None:
        tmp_dir = tempfile.mkdtemp(prefix='vt_db')
        print 'creating tmpdir:', tmp_dir

    db_connection = io.open_db_connection(config)
    obj_id_lists = {}
    filenames = []
    if restore:
        for dirpath, dirname, files in os.walk(restore):
            for fname in files:
                if fname.endswith('.vt'):
                    filenames.append(os.path.join(restore, fname))
    else:
        for obj_type in obj_types:
            obj_id_lists[obj_type] = io.get_db_object_list(config, obj_type)

        # read data out of database
        thumbnail_dir = os.path.join(tmp_dir, 'thumbs')
        os.mkdir(thumbnail_dir)
        for obj_type, obj_ids in obj_id_lists.iteritems():
            for (obj_id, _, _) in obj_ids:
                old_version = io.get_db_object_version(db_connection, obj_id, 
                                                       'vistrail')

                print 'getting', obj_type, 'id', obj_id
                local_tmp_dir = os.path.join(tmp_dir, str(obj_id))
                vt_name = os.path.join(tmp_dir, str(obj_id) + '.vt')
                try:
                    res = obj_types[obj_type](obj_type, db_connection, obj_id,
                                          thumbnail_dir)
                    filenames.append(vt_name)
                    os.mkdir(local_tmp_dir)
                except:
                    import traceback
                    print "Could not read:", traceback.format_exc()
                    continue
                io.save_vistrail_bundle_to_zip_xml(res, vt_name, local_tmp_dir)

        # drop the old database
        # recreate with the new version of the specs
        io.setup_db_tables(db_connection, None, old_version)

    # add the new data back
    for filename in filenames:
        (res, _) = io.open_vistrail_bundle_from_zip_xml(filename)
        try:
            io.save_vistrail_bundle_to_db(res, db_connection, 'with_ids')
        except Exception, e:
            import traceback
            print filename, e, traceback.format_exc()
    io.close_db_connection(db_connection)

if __name__ == '__main__':
    import vistrails.core.application

    more_options = {'v:': ('set new schema version', False, 'version'),
                    'd:': ('set temporary directory', False, 'directory'),
                    'e:': ('restore from directory', False, 'restore')
                    }
    config, options = parse_db_cmd_line(sys.argv, more_options)
    new_version = None
    if options['v']:
        new_version = options['v']

    vistrails.core.application.init()

    update_db(config, new_version, options['d'], options['e']) 



###############################################################################
##
## Copyright (C) 2014-2015, New York University.
## Copyright (C) 2011-2014, NYU-Poly.
## Copyright (C) 2006-2011, University of Utah.
## All rights reserved.
## Contact: contact@vistrails.org
##
## This file is part of VisTrails.
##
## "Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions are met:
##
##  - Redistributions of source code must retain the above copyright notice,
##    this list of conditions and the following disclaimer.
##  - Redistributions in binary form must reproduce the above copyright
##    notice, this list of conditions and the following disclaimer in the
##    documentation and/or other materials provided with the distribution.
##  - Neither the name of the New York University nor the names of its
##    contributors may be used to endorse or promote products derived from
##    this software without specific prior written permission.
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
##
###############################################################################

############################################################################
# web browser view implementation
############################################################################
from __future__ import division

from PyQt4 import QtCore, QtGui, QtWebKit
from vistrails.packages.spreadsheet.basic_widgets import SpreadsheetCell
from vistrails.packages.spreadsheet.spreadsheet_cell import QCellWidget, \
    QCellToolBar
import os
import shutil
############################################################################

class WebViewCell(SpreadsheetCell):
    """
    WebViewCell is a custom Module to view URLs using WebKit
    
    """
    def compute(self):
        """ compute() -> None
        Dispatch the URL to the spreadsheet
        """
        if self.has_input("url"):
            urlValue = self.get_input("url")
            fileValue = None
        elif self.has_input("file"):
            fileValue = self.get_input("file")
            urlValue = None
        else:
            fileValue = None
            urlValue = None
        self.displayAndWait(WebViewCellWidget, (urlValue, fileValue))

class WebViewCellWidget(QCellWidget):
    """
    WebViewCellWidget has a QTextBrowser to display HTML files
    
    """
    save_formats = QCellWidget.save_formats + ["HTML files (*.html)"]

    def __init__(self, parent=None):
        """ WebViewCellWidget(parent: QWidget) -> WebViewCellWidget
        Create a rich text cell without a toolbar
        
        """
        QCellWidget.__init__(self, parent)
        self.setLayout(QtGui.QVBoxLayout(self))
        QtWebKit.QWebSettings.globalSettings().setAttribute(
            QtWebKit.QWebSettings.DeveloperExtrasEnabled, True)
        self.browser = QtWebKit.QWebView()
        self.layout().addWidget(self.browser)
        self.browser.setMouseTracking(True)
        self.urlSrc = None

    def updateContents(self, inputPorts):
        """ updateContents(inputPorts: tuple) -> None
        Updates the contents with a new changed in filename
        
        """
        self.urlSrc = None
        (urlValue, fileValue) = inputPorts
        if urlValue:
            url = QtCore.QUrl(urlValue)
            self.browser.load(url)
            self.urlSrc = url
        elif fileValue:
            url = QtCore.QUrl.fromLocalFile(fileValue.name)
            self.browser.load(url)
            self.urlSrc = url
        else:
            self.browser.setHtml("No HTML file is specified!")

    def dumpToFile(self, filename):
        if os.path.splitext(filename)[1].lower() in ('.html', '.htm'):
            with open(filename, 'wb') as fp:
                fp.write(self.browser.page().mainFrame().toHtml())
        else:
            super(WebViewCellWidget, self).dumpToFile(filename)

    def saveToPDF(self, filename):
        printer = QtGui.QPrinter()
        printer.setOutputFormat(QtGui.QPrinter.PdfFormat)
        printer.setOutputFileName(filename)
        self.browser.print_(printer)



###############################################################################
##
## Copyright (C) 2014-2015, New York University.
## Copyright (C) 2011-2014, NYU-Poly.
## Copyright (C) 2006-2011, University of Utah.
## All rights reserved.
## Contact: contact@vistrails.org
##
## This file is part of VisTrails.
##
## "Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions are met:
##
##  - Redistributions of source code must retain the above copyright notice,
##    this list of conditions and the following disclaimer.
##  - Redistributions in binary form must reproduce the above copyright
##    notice, this list of conditions and the following disclaimer in the
##    documentation and/or other materials provided with the distribution.
##  - Neither the name of the New York University nor the names of its
##    contributors may be used to endorse or promote products derived from
##    this software without specific prior written permission.
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
##
###############################################################################
from __future__ import division

from xml.dom import minidom
import xml.parsers.expat
import __builtin__

def eval_xml_value(node):
    """eval_xml_value(node) -> value

       evaluates an xml node as the following examples:

       <str value='foo'/> -> 'foo'
       <int value='3'/> -> 3
       <float value='3.141592'> -> 3.141592
       <bool value='False'> -> False
    """
    
    key_name = node.nodeName
    type_ = getattr(__builtin__, key_name)
    str_value = str(node.attributes['value'].value)

    # Tricky case bool('False') == True
    if type_ == bool:
        if str_value == 'True':
            return True
        elif str_value == 'False':
            return False
        else:
            raise ValueError("eval_xml_value: Bogus bool value '%s'" % str_value)
    return type_(str_value)

def quote_xml_value(dom, value):
    """quote_xml_value(dom, value) -> value

       quotes a value as an xml node so that
       eval_xml_value(quote_xml_value(dom, value)) == value

       <str value='foo'/> <- 'foo'
       <int value='3'/> <- 3
       <float value='3.141592'> <- 3.141592
       <bool value='False'> <- False
    """

    el = dom.createElement(type(value).__name__)
    el.setAttribute('value', str(value))
    return el

def named_elements(element, elname):
    """named_elements(element, elname) -> Node 
    Helper function that iterates over the element child Nodes searching
    for node with name elname.

    """
    for node in element.childNodes:
        if node.nodeName == elname:
            yield node

def enter_named_element(element, elname):
    """enter_named_element(element, elname) -> Node 
    Returns first child of element with name elname

    """
    for node in named_elements(element, elname):
        return node
    return None

def elements_filter(element, element_predicate):
    """elements_filter(element, element_predicate) -> Node iterator
    Helper function that iterates over the element child Nodes searching
    for nodes that pass element_predicate, that is, node for which

    element_predicate(node) == True

    """
    for node in element.childNodes:
        if element_predicate(node):
            yield node

class XMLWrapper(object):
    """Helper to parse a general XML file. It provides functions to open and 
    close files.
    It must be subclassed to parse specifi files. """

    class XMLParseError(Exception):

       
        def __init__(self, line, char, code):
            self._line = line
            self._char = char
            self._code = code
            
        def __str__(self):
            return ("XML Parse error at line %s, col %s: %s" %
                    (self._line,
                     self._char,
                     xml.parsers.expat.ErrorString(self._code)))

    def open_file(self, filename):
        """open_file(filename: str) -> None 
        Parses an XML file.

        """
        self.filename = filename
        try:
            self.dom = minidom.parse(filename)
        except xml.parsers.expat.ExpatError, e:
            raise self.XMLParseError(e.lineno, e.offset, e.code) 

    def create_document_from_string(self, text):
        """parse_string(text:str) -> dom
        Parses an xml string and returns the DOM object

        """
        try:
            dom = minidom.parseString(text)
        except xml.parsers.expat.ExpatError, e:
            raise self.XMLParseError(e.lineno, e.offset, e.code)
        return dom
    
    def close_file(self):
        """close_file() -> None 
        Removes the association with the XML file loaded by open_file 
        method. 

        """
        if self.dom:
            self.dom.unlink()
        self.filename = None
        self.dom = None
    
    def create_document(self, nodename):
        """create_document(nodename: str) -> xml element 
        Creates a documentElement 
        
        """
        impl = minidom.getDOMImplementation()
        dom = impl.createDocument(None, nodename, None)
        return dom

    def write_document(self, root, filename):
        """write_document(root:xml element, filename: str) -> None
        Save as an XML file 
        
        """
        output_file = open(filename,'w')
        root.writexml(output_file, "  ", "  ", '\n')
        output_file.close()

    def __str__(self):
        """ __str__() -> str 
        Returns the XML that self.dom represents as a string 
        
        """
        return self.dom.toprettyxml()

################################################################################
# Testing

import unittest

class TestXmlUtils(unittest.TestCase):
    def test_named_elements(self):
        """ Exercises searching for elements """
        xmlStr = """<root> 
                        <child>
                            <grandchild></grandchild>
                            <grandchild></grandchild>
                         </child>
                         <child></child>
                     </root>"""
        dom = minidom.parseString(xmlStr)
        root = dom.documentElement
        childcount = 0
        for node in named_elements(root,'child'):
            childcount += 1
        self.assertEquals(childcount,2)
        
        grandchildcount = 0
        for node in named_elements(root,'grandchild'):
            grandchildcount += 1
        self.assertEquals(grandchildcount,0)

    def test_eval_quote(self):
        xmlStr = """<root> 
                        <child>
                            <grandchild></grandchild>
                            <grandchild></grandchild>
                         </child>
                         <child></child>
                     </root>"""
        dom = minidom.parseString(xmlStr)
        def do_it_1(v):
            q = quote_xml_value(dom, v)
            v2 = eval_xml_value(q)
            self.assertEquals(v, v2)
        def do_it_2(q):
            q = minidom.parseString(q).documentElement
            v = eval_xml_value(q)
            self.assertEquals(q.toxml(), quote_xml_value(dom, v).toxml())
        do_it_1(2)
        do_it_1(3.0)
        do_it_1(False)
        do_it_1(True)
        do_it_1('Foobar')
        do_it_1('with<brackets>')

        do_it_2('<str value="Foo"/>')
        do_it_2('<bool value="False"/>')
        do_it_2('<bool value="True"/>')
        do_it_2('<int value="3"/>')
        do_it_2('<float value="4.0"/>')
        
if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

# This example demonstrates how to use the vtkLineWidget to seed and
# manipulate streamlines. Two line widgets are created. One is invoked
# by pressing 'i', the other by pressing 'L' (capital). Both can exist
# together.

import vtk
from vtk.util.misc import vtkGetDataRoot
VTK_DATA_ROOT = vtkGetDataRoot()

# Start by loading some data.
pl3d = vtk.vtkPLOT3DReader()
pl3d.SetXYZFileName(VTK_DATA_ROOT + "/Data/combxyz.bin")
pl3d.SetQFileName(VTK_DATA_ROOT + "/Data/combq.bin")
pl3d.SetScalarFunctionNumber(100)
pl3d.SetVectorFunctionNumber(202)
pl3d.Update()

# The line widget is used seed the streamlines.
lineWidget = vtk.vtkLineWidget()
seeds = vtk.vtkPolyData()
lineWidget.SetInput(pl3d.GetOutput())
lineWidget.SetAlignToYAxis()
lineWidget.PlaceWidget()
lineWidget.GetPolyData(seeds)
lineWidget.ClampToBoundsOn()

rk4 = vtk.vtkRungeKutta4()
streamer = vtk.vtkStreamLine()
streamer.SetInputConnection(pl3d.GetOutputPort())
streamer.SetSource(seeds)
streamer.SetMaximumPropagationTime(100)
streamer.SetIntegrationStepLength(.2)
streamer.SetStepLength(.001)
streamer.SetNumberOfThreads(1)
streamer.SetIntegrationDirectionToForward()
streamer.VorticityOn()
streamer.SetIntegrator(rk4)
rf = vtk.vtkRibbonFilter()
rf.SetInputConnection(streamer.GetOutputPort())
rf.SetWidth(0.1)
rf.SetWidthFactor(5)
streamMapper = vtk.vtkPolyDataMapper()
streamMapper.SetInputConnection(rf.GetOutputPort())
streamMapper.SetScalarRange(pl3d.GetOutput().GetScalarRange())
streamline = vtk.vtkActor()
streamline.SetMapper(streamMapper)
streamline.VisibilityOff()

# The second line widget is used seed more streamlines.
lineWidget2 = vtk.vtkLineWidget()
seeds2 = vtk.vtkPolyData()
lineWidget2.SetInput(pl3d.GetOutput())
lineWidget2.PlaceWidget()
lineWidget2.GetPolyData(seeds2)
lineWidget2.SetKeyPressActivationValue('L')

streamer2 = vtk.vtkStreamLine()
streamer2.SetInputConnection(pl3d.GetOutputPort())
streamer2.SetSource(seeds2)
streamer2.SetMaximumPropagationTime(100)
streamer2.SetIntegrationStepLength(.2)
streamer2.SetStepLength(.001)
streamer2.SetNumberOfThreads(1)
streamer2.SetIntegrationDirectionToForward()
streamer2.VorticityOn()
streamer2.SetIntegrator(rk4)
rf2 = vtk.vtkRibbonFilter()
rf2.SetInputConnection(streamer2.GetOutputPort())
rf2.SetWidth(0.1)
rf2.SetWidthFactor(5)
streamMapper2 = vtk.vtkPolyDataMapper()
streamMapper2.SetInputConnection(rf2.GetOutputPort())
streamMapper2.SetScalarRange(pl3d.GetOutput().GetScalarRange())
streamline2 = vtk.vtkActor()
streamline2.SetMapper(streamMapper2)
streamline2.VisibilityOff()

outline = vtk.vtkStructuredGridOutlineFilter()
outline.SetInputConnection(pl3d.GetOutputPort())
outlineMapper = vtk.vtkPolyDataMapper()
outlineMapper.SetInputConnection(outline.GetOutputPort())
outlineActor = vtk.vtkActor()
outlineActor.SetMapper(outlineMapper)

# Create the RenderWindow, Renderer and both Actors
ren = vtk.vtkRenderer()
renWin = vtk.vtkRenderWindow()
renWin.AddRenderer(ren)
iren = vtk.vtkRenderWindowInteractor()
iren.SetRenderWindow(renWin)

# Callback functions that actually generate streamlines.
def BeginInteraction(obj, event):
    global streamline
    streamline.VisibilityOn() 

def GenerateStreamlines(obj, event):
    global seeds, renWin
    obj.GetPolyData(seeds)
    renWin.Render()

def BeginInteraction2(obj, event):
    global streamline2
    streamline2.VisibilityOn()

def GenerateStreamlines2(obj, event):
    global seeds2, renWin
    obj.GetPolyData(seeds2)
    renWin.Render()

# Associate the line widget with the interactor and setup callbacks.
lineWidget.SetInteractor(iren)
lineWidget.AddObserver("StartInteractionEvent", BeginInteraction)
lineWidget.AddObserver("InteractionEvent", GenerateStreamlines)

lineWidget2.SetInteractor(iren)
lineWidget2.AddObserver("StartInteractionEvent", BeginInteraction2)
lineWidget2.AddObserver("EndInteractionEvent", GenerateStreamlines2)

# Add the actors to the renderer, set the background and size
ren.AddActor(outlineActor)
ren.AddActor(streamline)
ren.AddActor(streamline2)

ren.SetBackground(1, 1, 1)
renWin.SetSize(300, 300)
ren.SetBackground(0.1, 0.2, 0.4)

cam1 = ren.GetActiveCamera()
cam1.SetClippingRange(3.95297, 50)
cam1.SetFocalPoint(9.71821, 0.458166, 29.3999)
cam1.SetPosition(2.7439, -37.3196, 38.7167)
cam1.SetViewUp(-0.16123, 0.264271, 0.950876)

iren.Initialize()
renWin.Render()
iren.Start()



#!/usr/bin/env python
###############################################################################
##
## Copyright (C) 2014-2015, New York University.
## Copyright (C) 2011-2014, NYU-Poly.
## Copyright (C) 2006-2011, University of Utah.
## All rights reserved.
## Contact: contact@vistrails.org
##
## This file is part of VisTrails.
##
## "Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions are met:
##
##  - Redistributions of source code must retain the above copyright notice,
##    this list of conditions and the following disclaimer.
##  - Redistributions in binary form must reproduce the above copyright
##    notice, this list of conditions and the following disclaimer in the
##    documentation and/or other materials provided with the distribution.
##  - Neither the name of the New York University nor the names of its
##    contributors may be used to endorse or promote products derived from
##    this software without specific prior written permission.
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
##
###############################################################################
"""Main file for running VisTrails in server mode"""
import os
import sys

def fix_paths():
    import site
    if not hasattr(site, "USER_BASE"): return # We are running py2app

    # Fix import path: add parent directory(so that we can
    # import vistrails.[gui|...] and remove other paths below it (we might have
    # been started from a subdir)
    # A better solution is probably to move run.py up a
    # directory in the repo
    old_dir = os.path.realpath(os.path.dirname(__file__))
    vistrails_dir = os.path.realpath(os.path.join(os.path.dirname(__file__), '..'))
    i = 0
    while i < len(sys.path):
        rpath = os.path.realpath(sys.path[i])
        if rpath.startswith(old_dir):
            del sys.path[i]
        else:
            i += 1
    if vistrails_dir not in sys.path:
        sys.path.insert(0, vistrails_dir)

if __name__ == '__main__':
    fix_paths()

    # Set locale to English
    import locale
    locale.setlocale(locale.LC_ALL, 'C')

    import vistrails.gui.requirements
    vistrails.gui.requirements.require_pyqt4_api2()

    import vistrails.gui.application_server
    try:
        optionsDict = {
            'interactiveMode': False,
            'enablePackagesSilently': False,
            'handlerDontAsk': True,
        }
        v = vistrails.gui.application_server.start_server(optionsDict,
                                                          args=sys.argv[1:])
        app = vistrails.gui.application_server.VistrailsServer()
    except SystemExit, e:
        print str(e)
        sys.exit(e)
    except Exception, e:
        import traceback
        print "Uncaught exception on initialization: %s" % (
                traceback._format_final_exc_line(type(e).__name__, e))
        traceback.print_exc()
        sys.exit(255)
     
    v = app.run_server()
    vistrails.gui.application_server.stop_server()
    sys.exit(v)



#!/usr/bin/env python

# This example shows how to extract portions of an unstructured grid
# using vtkExtractUnstructuredGrid. vtkConnectivityFilter is also used
# to extract connected components.
#
# The data found here represents a blow molding process. Blow molding
# requires a mold and parison (hot, viscous plastic) which is shaped
# by the mold into the final form. The data file contains several steps
# in time for the analysis.

import vtk
from vtk.util.misc import vtkGetDataRoot
VTK_DATA_ROOT = vtkGetDataRoot()

# Create a reader to read the unstructured grid data. We use a
# vtkDataSetReader which means the type of the output is unknown until
# the data file is read. So we follow the reader with a
# vtkCastToConcrete and cast the output to vtkUnstructuredGrid.
reader = vtk.vtkDataSetReader()
reader.SetFileName(VTK_DATA_ROOT + "/Data/blow.vtk")
reader.SetScalarsName("thickness9")
reader.SetVectorsName("displacement9")
castToUnstructuredGrid = vtk.vtkCastToConcrete()
castToUnstructuredGrid.SetInputConnection(reader.GetOutputPort())
warp = vtk.vtkWarpVector()
warp.SetInput(castToUnstructuredGrid.GetUnstructuredGridOutput())

# The connectivity filter extracts the first two regions. These are
# know to represent the mold.
connect = vtk.vtkConnectivityFilter()
connect.SetInputConnection(warp.GetOutputPort())
connect.SetExtractionModeToSpecifiedRegions()
connect.AddSpecifiedRegion(0)
connect.AddSpecifiedRegion(1)
moldMapper = vtk.vtkDataSetMapper()
moldMapper.SetInputConnection(reader.GetOutputPort())
moldMapper.ScalarVisibilityOff()
moldActor = vtk.vtkActor()
moldActor.SetMapper(moldMapper)
moldActor.GetProperty().SetColor(.2, .2, .2)
moldActor.GetProperty().SetRepresentationToWireframe()

# Another connectivity filter is used to extract the parison.
connect2 = vtk.vtkConnectivityFilter()
connect2.SetInputConnection(warp.GetOutputPort())
connect2.SetExtractionModeToSpecifiedRegions()
connect2.AddSpecifiedRegion(2)

# We use vtkExtractUnstructuredGrid because we are interested in
# looking at just a few cells. We use cell clipping via cell id to
# extract the portion of the grid we are interested in.
extractGrid = vtk.vtkExtractUnstructuredGrid()
extractGrid.SetInputConnection(connect2.GetOutputPort())
extractGrid.CellClippingOn()
extractGrid.SetCellMinimum(0)
extractGrid.SetCellMaximum(23)
parison = vtk.vtkGeometryFilter()
parison.SetInputConnection(extractGrid.GetOutputPort())
normals2 = vtk.vtkPolyDataNormals()
normals2.SetInputConnection(parison.GetOutputPort())
normals2.SetFeatureAngle(60)
lut = vtk.vtkLookupTable()
lut.SetHueRange(0.0, 0.66667)
parisonMapper = vtk.vtkPolyDataMapper()
parisonMapper.SetInputConnection(normals2.GetOutputPort())
parisonMapper.SetLookupTable(lut)
parisonMapper.SetScalarRange(0.12, 1.0)
parisonActor = vtk.vtkActor()
parisonActor.SetMapper(parisonMapper)

# graphics stuff
ren = vtk.vtkRenderer()
renWin = vtk.vtkRenderWindow()
renWin.AddRenderer(ren)
iren = vtk.vtkRenderWindowInteractor()
iren.SetRenderWindow(renWin)

# Add the actors to the renderer, set the background and size
ren.AddActor(parisonActor)
ren.AddActor(moldActor)
ren.SetBackground(1, 1, 1)
ren.ResetCamera()
ren.GetActiveCamera().Azimuth(60)
ren.GetActiveCamera().Roll(-90)
ren.GetActiveCamera().Dolly(2)
ren.ResetCameraClippingRange()
renWin.SetSize(500, 375)

iren.Initialize()
renWin.Render()
iren.Start()



###############################################################################
##
## Copyright (C) 2014-2015, New York University.
## Copyright (C) 2011-2014, NYU-Poly.
## Copyright (C) 2006-2011, University of Utah.
## All rights reserved.
## Contact: contact@vistrails.org
##
## This file is part of VisTrails.
##
## "Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions are met:
##
##  - Redistributions of source code must retain the above copyright notice,
##    this list of conditions and the following disclaimer.
##  - Redistributions in binary form must reproduce the above copyright
##    notice, this list of conditions and the following disclaimer in the
##    documentation and/or other materials provided with the distribution.
##  - Neither the name of the New York University nor the names of its
##    contributors may be used to endorse or promote products derived from
##    this software without specific prior written permission.
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
##
###############################################################################
from __future__ import division

from version import SearchCompiler
from visual import VisualQuery

class CombinedSearch(VisualQuery):
    def __init__(self, search_str=None, pipeline=None, versions_to_check=None,
                 use_regex=False):
        VisualQuery.__init__(self, pipeline, versions_to_check)
        self.search_str = search_str
        self.use_regex = use_regex

    def run(self, vistrail, name):
        VisualQuery.run(self, vistrail, name)
        compiler = SearchCompiler(self.search_str, self.use_regex)
        self.search_stmt = compiler.searchStmt

    def match(self, vistrail, action):
        if self.queryPipeline is not None and \
                len(self.queryPipeline.modules) > 0:
            if action.timestep in self.versionDict:
                return self.search_stmt.match(vistrail, action)
            return False
        else:
            return self.search_stmt.match(vistrail, action)

    def matchModule(self, version_id, module):
        if self.queryPipeline is not None and \
                len(self.queryPipeline.modules) > 0:
            return VisualQuery.matchModule(self, version_id, module)
        return True
    
    def getResultEntity(self, vistrail, versions_to_check):
        from vistrails.core.collection.vistrail import VistrailEntity

        locators = []
        vistrail_entity = None
        for version in versions_to_check:
            if version in vistrail.actionMap:
                action = vistrail.actionMap[version]
                if self.match(vistrail, action):
                    # have a match, get vistrail entity
                    if vistrail_entity is None:
                        vistrail_entity = VistrailEntity()
                        # don't want to add all workflows, executions
                        vistrail_entity.set_vistrail(vistrail)
                    vistrail_entity.add_workflow_entity(version)
                    # FIXME this is not done at the low level but in
                    # Collection class, probably should be reworked
                    vistrail_entity.wf_entity_map[version].parent = \
                        vistrail_entity
        return vistrail_entity



#!/usr/bin/env python

import itertools
import sys

import vistrails.core.application
from vistrails.core.api import Package
from vistrails.core.modules.module_registry import get_module_registry

heading_order = ['*', '=', '^', '-', '"', '+', '#']

def trim_docstring(docstring):
    """Copied from PEP 257: http://www.python.org/dev/peps/pep-0257/"""
    if not docstring:
        return ''
    # Convert tabs to spaces (following the normal Python rules)
    # and split into a list of lines:
    lines = docstring.expandtabs().splitlines()
    # Determine minimum indentation (first line doesn't count):
    indent = sys.maxint
    for line in lines[1:]:
        stripped = line.lstrip()
        if stripped:
            indent = min(indent, len(line) - len(stripped))
    # Remove indentation (first line is special):
    trimmed = [lines[0].strip()]
    if indent < sys.maxint:
        for line in lines[1:]:
            trimmed.append(line[indent:].rstrip())
    # Strip off trailing and leading blank lines:
    while trimmed and not trimmed[-1]:
        trimmed.pop()
    while trimmed and not trimmed[0]:
        trimmed.pop(0)
    # Return a single string:
    return '\n'.join(trimmed)

def indent_docstring(docstring, indent=0):
    new_docstring = ""
    for line in docstring.splitlines():
        new_docstring += (" " * indent) + line + "\n"
    return new_docstring

def format_docstring(docstring, indent=0):
    return indent_docstring(trim_docstring(docstring), indent)

def get_examples(desc):
    return []

def generate_module_doc(desc, f=None, depth=1, inherited=True):
    reg = get_module_registry()
    print >>f, desc.name
    print >>f, heading_order[depth] * len(desc.name)
    print >>f, ""
    print >>f, ".. py:class:: %s" % desc.name
    print >>f, ""
    if desc.module.__doc__:
        print >>f, format_docstring(desc.module.__doc__, 2)
        print >>f, ""

    if inherited:
        input_ports = reg.module_destination_ports_from_descriptor(True, desc)
        output_ports = reg.module_source_ports_from_descriptor(True, desc)
    else:
        input_ports = reg.module_ports('input', desc).values()
        input_ports.sort(key=lambda x: (x.sort_key, x.id))
        output_ports = reg.module_ports('output', desc).values()
        output_ports.sort(key=lambda x: (x.sort_key, x.id))
        
    if len(input_ports) > 0:
        print >>f, "  *Input Ports*"
        for port in input_ports:
            sigstring = port.sigstring.replace(':', '.')[1:-1]
            print >>f, "    .. py:attribute:: %s" % port.name
            print >>f, ""            
            print >>f, "      | *Signature*: :py:class:`%s`" % sigstring
            if port.docstring():
                print >>f, "      | *Description*: %s" % format_docstring(port.docstring(), 9)
            print >>f, ""
    if len(output_ports) > 0:
        print >>f, "  *Output Ports*"
        for port in output_ports:
            sigstring = port.sigstring.replace(':', '.')[1:-1]
            print >>f, "    .. py:attribute:: %s" % port.name
            print >>f, ""            
            print >>f, "      | *Signature*: :py:class:`%s`" % sigstring
            if port.docstring():
                print >>f, "      | *Description*: %s" % format_docstring(port.docstring(), 9)
            print >>f, ""

    examples = get_examples(desc)
    if len(examples) > 0:
        print >>f, "  *Examples*"
        for example in examples:
            print >>f, "   * %s" % example
        print >>f, ""

def generate_docs(pkg, namespace=None, f=None):
    print >>f, pkg._package.name
    print >>f, heading_order[0] * len(pkg._package.name)
    print >>f, ""
    print >>f, ".. py:module:: %s\n" % pkg._package.identifier
    print >>f, '| *Identifier*: %s' % pkg._package.identifier
    print >>f, '| *Version*: %s\n' % pkg._package.version
    print >>f, format_docstring(pkg._package.description, 0)
    print >>f, ""
    
    if namespace == '':
        for desc in pkg._namespaces[1]:
            generate_module_doc(desc, f, 1)
    else:
        if namespace is None:
            namespace = ''
            for desc in pkg._namespaces[1]:
                generate_module_doc(desc, f)
            namespaces = sorted(item + (1,) for item in 
                                pkg._namespaces[0].iteritems())
        else:
            namespace_dict = pkg._namespaces[0]
            descs = pkg._namespaces[1]
            split_ns = namespace.split('|')
            for i, ns in enumerate(split_ns):
                print >>f, ns
                print >>f, heading_order[i+1] * len(ns)
                print >>f, ""
                (namespace_dict, descs) = namespace_dict[ns]
            namespaces = [(namespace, (namespace_dict, descs), len(split_ns))]
        
        for (ns, (child_namespaces, descs), depth) in namespaces:
            print >>f, ns
            print >>f, heading_order[depth] * len(ns)
            print >>f, ""
            for desc in descs:
                generate_module_doc(desc, f, depth+1)
            namespaces = \
                itertools.chain([(ns + '|' + c[0], c[1]) 
                                 for c in child_namespaces.iteritems()],
                                namespaces, depth+1)

def run():
    vistrails.core.application.init()
    pkg = Package("org.vistrails.vistrails.basic")
    # pkg = Package("org.vistrails.vistrails.matplotlib")
    generate_docs(pkg)

if __name__ == '__main__':
    run()



#!/usr/bin/env python

# This script shows how to manually create a vtkPolyData with a
# triangle strip.

import vtk

# First we'll create some points.
points = vtk.vtkPoints()
points.InsertPoint(0, 0.0, 0.0, 0.0)
points.InsertPoint(1, 0.0, 1.0, 0.0)
points.InsertPoint(2, 1.0, 0.0, 0.0)
points.InsertPoint(3, 1.0, 1.0, 0.0)
points.InsertPoint(4, 2.0, 0.0, 0.0)
points.InsertPoint(5, 2.0, 1.0, 0.0)
points.InsertPoint(6, 3.0, 0.0, 0.0)
points.InsertPoint(7, 3.0, 1.0, 0.0)

# The cell array can be thought of as a connectivity list.  Here we
# specify the number of points followed by that number of point
# ids. This can be repeated as many times as there are primitives in
# the list.
strips = vtk.vtkCellArray()
strips.InsertNextCell(8) # number of points
strips.InsertCellPoint(0)
strips.InsertCellPoint(1)
strips.InsertCellPoint(2)
strips.InsertCellPoint(3)
strips.InsertCellPoint(4)
strips.InsertCellPoint(5)
strips.InsertCellPoint(6)
strips.InsertCellPoint(7)
profile = vtk.vtkPolyData()
profile.SetPoints(points)
profile.SetStrips(strips)

map = vtk.vtkPolyDataMapper()
map.SetInput(profile)

strip = vtk.vtkActor()
strip.SetMapper(map)
strip.GetProperty().SetColor(0.3800, 0.7000, 0.1600)

# Create the usual rendering stuff.
ren = vtk.vtkRenderer()
renWin = vtk.vtkRenderWindow()
renWin.AddRenderer(ren)
iren = vtk.vtkRenderWindowInteractor()
iren.SetRenderWindow(renWin)

# Add the actors to the renderer, set the background and size
ren.AddActor(strip)

ren.SetBackground(1, 1, 1)
renWin.SetSize(250, 250)

iren.Initialize()
renWin.Render()
iren.Start()



###############################################################################
##
## Copyright (C) 2014-2015, New York University.
## Copyright (C) 2011-2014, NYU-Poly.
## Copyright (C) 2006-2011, University of Utah.
## All rights reserved.
## Contact: contact@vistrails.org
##
## This file is part of VisTrails.
##
## "Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions are met:
##
##  - Redistributions of source code must retain the above copyright notice,
##    this list of conditions and the following disclaimer.
##  - Redistributions in binary form must reproduce the above copyright
##    notice, this list of conditions and the following disclaimer in the
##    documentation and/or other materials provided with the distribution.
##  - Neither the name of the New York University nor the names of its
##    contributors may be used to endorse or promote products derived from
##    this software without specific prior written permission.
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
##
###############################################################################

"""Utility functions for core.bundles"""
from __future__ import division

from vistrails.core import debug
import vistrails.core.system
import os
import platform
import sys

##############################################################################

def guess_graphical_sudo():
    """Tries to guess what to call to run a shell with elevated privileges.

    Returns: (sudo, escape)
    Where:
      sudo is the command to be used to gain root privileges, it 
           should contain %s where the actual command will be inserted
      escape is True if the rest of the line needs to be escaped
    """
    if sys.platform == 'win32':
        return '%s', False
    # sudo needs -E so that the Xauthority file is found and root can connect
    # to the user's X server
    if vistrails.core.system.executable_is_in_path('kdesudo'):
        return 'kdesudo %s', True
    elif vistrails.core.system.executable_is_in_path('kdesu'):
        return 'kdesu %s', False
    elif vistrails.core.system.executable_is_in_path('gksu'):
        return 'gksu %s', False
    elif (vistrails.core.system.executable_is_in_path('sudo') and
          vistrails.core.system.executable_is_in_path('zenity')):
        # This is a reasonably convoluted hack to only prompt for the password
        # if user has not recently entered it
        return ('((echo "" | sudo -v -S -p "") || '
                '(zenity --entry --title "sudo password prompt" --text '
                '"Please enter your password to give the system install '
                'authorization." --hide-text="" | sudo -v -S -p "")); '
                'sudo -E -S -p "" %s',
               False)
        # graphical sudo for osx
    elif vistrails.core.system.executable_is_in_path('osascript'):
        return "osascript -e " \
               "'do shell script %s with administrator privileges'", True
    else:
        debug.warning("Could not find a graphical sudo-like command.")

        if vistrails.core.system.executable_is_in_path('sudo'):
            debug.warning("Will use regular sudo")
            return "sudo -E %s", False
        else:
            debug.warning("Will use regular su")
            return "su --preserve-environment -c %s", True

##############################################################################

class System_guesser(object):

    def __init__(self):
        self._callable_dict = {}

    def add_test(self, test, system_name):
        if self._callable_dict.has_key(system_name):
            raise ValueError("test for '%s' already present." % system_name)
        if system_name == 'UNKNOWN':
            raise ValueError("Invalid system name")
        assert isinstance(system_name, str)
        self._callable_dict[system_name] = test

    def guess_system(self):
        for (name, callable_) in self._callable_dict.iteritems():
            if callable_():
                return name
        else:
            return 'UNKNOWN'

_system_guesser = System_guesser()

##############################################################################
# System tests

def _guess_suse():
    try:
        tokens = open('/etc/SuSE-release').readline()[-1].split()
        return tokens[0] == 'SUSE'
    except (IOError, IndexError):
        return False
_system_guesser.add_test(_guess_suse, 'linux-suse')

def _guess_ubuntu():
    return platform.linux_distribution()[0]=='Ubuntu' or \
           platform.linux_distribution()[0]=='LinuxMint'
_system_guesser.add_test(_guess_ubuntu, 'linux-ubuntu')

def _guess_debian():
    return platform.linux_distribution()[0].lower() == 'debian'
_system_guesser.add_test(_guess_debian, 'linux-debian')

def _guess_fedora():
    return os.path.isfile('/etc/fedora-release')
_system_guesser.add_test(_guess_fedora, 'linux-fedora')

def _guess_windows():
    return vistrails.core.system.systemType == 'Windows'
_system_guesser.add_test(_guess_windows, 'windows')

##############################################################################

def guess_system():
    """guess_system will try to identify which system you're
    running. Result will be a string describing the system. This is
    more discriminating than Linux/OSX/Windows: We'll try to figure
    out whether you're running SuSE, Debian, Ubuntu, RedHat, fink,
    darwinports, etc.

    Currently, we only support SuSE, Debian, Ubuntu and
    Fedora. However, we only have actual bundle installing for Debian,
    Ubuntu and Fedora.

    """
    return _system_guesser.guess_system()



###############################################################################
##
## Copyright (C) 2014-2015, New York University.
## Copyright (C) 2011-2014, NYU-Poly.
## Copyright (C) 2006-2011, University of Utah.
## All rights reserved.
## Contact: contact@vistrails.org
##
## This file is part of VisTrails.
##
## "Redistribution and use in source and binary forms, with or without
## modification, are permitted provided that the following conditions are met:
##
##  - Redistributions of source code must retain the above copyright notice,
##    this list of conditions and the following disclaimer.
##  - Redistributions in binary form must reproduce the above copyright
##    notice, this list of conditions and the following disclaimer in the
##    documentation and/or other materials provided with the distribution.
##  - Neither the name of the New York University nor the names of its
##    contributors may be used to endorse or promote products derived from
##    this software without specific prior written permission.
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
## THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
## CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
## EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
## PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
## OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
## WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
## OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
## ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
##
###############################################################################
from __future__ import division

from PyQt4 import QtCore, QtGui
import os

from vistrails.core.db.locator import FileLocator, DBLocator, UntitledLocator
from vistrails.core.publishing.parse_latex import parse_latex_file, \
     parse_vt_command, build_vt_command
from vistrails.gui.common_widgets import QDockPushButton
from vistrails.gui.vistrails_palette import QVistrailsPaletteInterface

class QLatexFigureItem(QtGui.QListWidgetItem):
    def __init__(self, opt_dict, parent=None):
        QtGui.QListWidgetItem.__init__(self, parent)
        self.opt_dict = opt_dict

    def update_opt_dict(self, opt_dict):
        self.opt_dict = opt_dict

    def get_opt_dict(self):
        return self.opt_dict


class QLatexAssistant(QtGui.QWidget, QVistrailsPaletteInterface):
    def __init__(self, parent=None, f=QtCore.Qt.WindowFlags()):
        QtGui.QWidget.__init__(self, parent, f)
        
        self.set_title("Export To LaTeX")

        # add button to select tex source
        # listview to select figure 
        # thumbnail display
        # set figure specific options (show workflow, show execution, show
        # set includegraphics options
        source_label = QtGui.QLabel("LaTeX Source:")
        self.source_edit = QtGui.QLineEdit()
        source_selector = QDockPushButton("Select...")
        # source_selector = QtGui.QToolButton()
        # source_selector.setIcon(QtGui.QIcon(
        #         self.style().standardPixmap(QtGui.QStyle.SP_DirOpenIcon)))
        # source_selector.setIconSize(QtCore.QSize(12,12))
        source_selector.setToolTip("Open a file chooser")
        # source_selector.setAutoRaise(True)
        self.connect(source_selector,
                     QtCore.SIGNAL('clicked()'),
                     self.selectSource)

        source_group = QtGui.QGroupBox("LaTeX Source")
        s_layout = QtGui.QHBoxLayout()
        s_layout.addWidget(source_label)
        s_layout.addWidget(self.source_edit)
        s_layout.addWidget(source_selector)
        s_layout.setStretch(1,1)
        source_group.setLayout(s_layout)

        self.figure_list = QtGui.QListWidget()
        self.figure_list.setSelectionMode(self.figure_list.SingleSelection)
        self.preview_image = QtGui.QLabel()
        self.preview_image.setScaledContents(False)
        self.preview_image.setMinimumSize(240, 240)
        add_figure = QDockPushButton("Add Figure")
        delete_figure = QDockPushButton("Delete Figure")
        
        self.connect(add_figure,
                     QtCore.SIGNAL("clicked()"),
                     self.addFigure)
        self.connect(delete_figure,
                     QtCore.SIGNAL("clicked()"),
                     self.deleteFigure)
        self.connect(self.figure_list,
                     QtCore.SIGNAL("itemSelectionChanged()"),
                     self.figureSelected)
        
        figure_group = QtGui.QGroupBox("Figures")
        figure_layout = QtGui.QGridLayout()
        figure_layout.addWidget(self.figure_list,0,0,1,2)
        figure_layout.addWidget(self.preview_image,0,2)
        figure_layout.addWidget(add_figure,1,0,QtCore.Qt.AlignRight)
        figure_layout.addWidget(delete_figure,1,1,QtCore.Qt.AlignRight)
        figure_group.setLayout(figure_layout)
        
        # figure type, vistrail reference (vt_locator), version (smart tag)
        # use current version
        self.figure_type = QtGui.QComboBox()
        self.figure_type.setEditable(False)
        # items = []
        # items << "Workflow Results" << "Workflow Graph" << "History Tree Graph";
        self.figure_type.addItems(["Workflow Results", "Workflow Graph",
                                   "Version Tree"])
        self.figure_ref = QtGui.QLineEdit()
        version_label = QtGui.QLabel("Version:")
        self.figure_version = QtGui.QLineEdit()
        tag_label = QtGui.QLabel("Tag:")
        self.figure_tag = QtGui.QComboBox()
        self.figure_tag.setEditable(True)
        self.figure_smart = QtGui.QCheckBox("Smart Tag")
        current_button = QDockPushButton("Use Current")
        
        self.connect(current_button, QtCore.SIGNAL("clicked()"),
                     self.useCurrent)

        graphicx_label = QtGui.QLabel("Arguments for includegraphics:")
        self.graphicx_edit = QtGui.QLineEdit()

        self.def_group = QtGui.QGroupBox("Figure Definition")
        def_layout = QtGui.QVBoxLayout()
        def_h_layout = QtGui.QHBoxLayout()
        def_h_layout.addWidget(self.figure_ref)
        def_h_layout.addWidget(self.figure_type)
        def_h_layout.setStretch(0,1)
        def_layout.addLayout(def_h_layout)
        def_h_layout = QtGui.QHBoxLayout()        
        def_h_layout.addWidget(version_label)
        def_h_layout.addWidget(self.figure_version)
        def_h_layout.addWidget(tag_label)
        def_h_layout.addWidget(self.figure_tag)
        def_h_layout.addWidget(self.figure_smart)
        def_h_layout.addWidget(current_button)
        def_h_layout.setStretch(3,1)
        def_layout.addLayout(def_h_layout)
        def_h_layout = QtGui.QHBoxLayout()
        def_h_layout.addWidget(graphicx_label)
        def_h_layout.addWidget(self.graphicx_edit)
        def_h_layout.setStretch(1,1)
        def_layout.addLayout(def_h_layout)
        self.def_group.setLayout(def_layout)

        self.chbPdf = QtGui.QCheckBox("As PDF")
        self.chbCache = QtGui.QCheckBox("Cache Images")
        self.chbLatexVTL = QtGui.QCheckBox("Include .vtl")

        self.chbWorkflow = QtGui.QCheckBox("Include Workflow")
        self.chbFullTree = QtGui.QCheckBox("Include Full Tree")
        self.chbFullTree.setEnabled(False)
        self.chbExecute = QtGui.QCheckBox("Execute Workflow")
        self.chbSpreadsheet = QtGui.QCheckBox("Show Spreadsheet Only")

        self.gbEmbedOpt = QtGui.QGroupBox("Embed Options")
        gblayout = QtGui.QGridLayout()
        gblayout.addWidget(self.chbPdf, 0, 0)
        gblayout.addWidget(self.chbCache, 0, 1)
        gblayout.addWidget(self.chbLatexVTL, 1, 0)
        self.gbEmbedOpt.setLayout(gblayout)

        self.gbDownOpt = QtGui.QGroupBox("Download Options")
        gblayout = QtGui.QGridLayout()
        gblayout.addWidget(self.chbWorkflow, 0, 0)
        gblayout.addWidget(self.chbFullTree, 0, 1)
        gblayout.addWidget(self.chbExecute, 1, 0)
        gblayout.addWidget(self.chbSpreadsheet, 1, 1)
        self.gbDownOpt.setLayout(gblayout)

        revert_button = QDockPushButton("Revert...")
        save_button = QDockPushButton("Save...")
        save_button.setAutoDefault(True)

        self.connect(save_button, QtCore.SIGNAL("clicked()"),
                     self.saveLatex)
        
        main_layout = QtGui.QVBoxLayout()
        main_layout.addWidget(source_group)
        main_layout.addWidget(figure_group)
        main_layout.addWidget(self.def_group)
        main_h_layout = QtGui.QHBoxLayout()
        main_h_layout.addWidget(self.gbEmbedOpt)
        main_h_layout.addWidget(self.gbDownOpt)
        main_layout.addLayout(main_h_layout)
        main_h_layout = QtGui.QHBoxLayout()
        main_h_layout.setAlignment(QtCore.Qt.AlignRight)
        main_h_layout.addWidget(revert_button)
        main_h_layout.addWidget(save_button)
        main_layout.addLayout(main_h_layout)
        
        self.setLayout(main_layout)

        self.texts = None
        self.selected_item = None

    def selectSource(self):
        fname = QtGui.QFileDialog.getOpenFileName(self,
                                                  'Load LaTeX File...',
                                                  self.source_edit.text(),
                                                  'LaTeX files (*.tex)')
        if fname:
            self.source_edit.setText(fname)
            self.texts = parse_latex_file(fname)
            for cmd_tuple in self.texts[1]:
                if cmd_tuple:
                    opt_dict = parse_vt_command(*cmd_tuple)
                    item = QLatexFigureItem(opt_dict)
                    item.setText("Figure %d" % (self.figure_list.count() + 1))
                    self.figure_list.addItem(item)

    def readImage(self, opt_dict):
        if 'pdf' in opt_dict:
            out_type = 'pdf'
        else:
            out_type = 'png'
        if 'showtree' in opt_dict:
            version_str = ''
        else:
            version_str = '_%s' % opt_dict['version']
        if 'showworkflow' in opt_dict:
            graph_str = '_graph'
        else:
            graph_str = ''

        if 'filename' in opt_dict:
            figure_name = "%s%s_%s%s" % (opt_dict['filename'], version_str,
                                         out_type, graph_str)
        else:
            figure_name = "%s_%s_%s_%s%s_%s%s" % (opt_dict['host'],
                                                  opt_dict['db'],
                                                  opt_dict['port'],
                                                  opt_dict['vtid'],
                                                  version_str,
                                                  out_type,
                                                  graph_str)
        
        source_dir = os.path.dirname(str(self.source_edit.text()))
        path_to_figure = os.path.join(source_dir, 'vistrails_images',
                                      figure_name)

        #print 'LOOKING FOR %s' % path_to_figure
        if os.path.exists(path_to_figure):
            fnames = os.listdir(path_to_figure)
            for fname in fnames:
                #print "FOUND", fname
                if fname.endswith('png'):
                    pixmap = QtGui.QPixmap(os.path.join(path_to_figure, fname))
                    if pixmap.width() > pixmap.height():
                        pixmap = pixmap.scaledToWidth(240,
                                                      QtCore.Qt.SmoothTransformation)
                    else:
                        pixmap = pixmap.scaledToHeight(240, 
                                                       QtCore.Qt.SmoothTransformation)
                    self.preview_image.setPixmap(pixmap)
            

    def doCheckLink(self, do_set=False, opt_dict=None):
        check_links = {'pdf': self.chbPdf,
                       'buildalways': (self.chbCache, True, True),
                       'getvtl': self.chbLatexVTL,
                       'execute': self.chbExecute,
                       'showspreadsheetonly': self.chbSpreadsheet,
                       'embedworkflow': self.chbWorkflow,
                       'includefulltree': self.chbFullTree,
                       'tag': self.figure_smart}

        if opt_dict is None:
            opt_dict = {}
        for k, v in check_links.iteritems():
            opt_set = k in opt_dict
            if isinstance(v, tuple):
                chb = v[0]
                chb_default = v[1]
                if len(v) > 2:
                    chb_rev = v[2]
                else:
                    chb_rev = False
            else:
                chb = v
                chb_rev = False
                chb_default = False

            if do_set:
                if opt_set:
                    chb.setChecked(not chb_rev)
                else:
                    chb.setChecked(chb_default)
            else:
                if not chb_rev and chb.isChecked():
                    opt_dict[k] = None
                elif chb_rev and not chb.isChecked():
                    opt_dict[k] = None

        return opt_dict

    def setFigureInfo(self, opt_dict):
        self.doCheckLink(True, opt_dict)

        if 'showworkflow' in opt_dict:
            self.figure_type.setCurrentIndex(1)
        elif 'showtree' in opt_dict:
            self.figure_type.setCurrentIndex(2)
        else:
            self.figure_type.setCurrentIndex(0)
            
        if 'version' in opt_dict:
            self.figure_version.setText(str(opt_dict['version']))
        else:
            self.figure_version.setText("")
        if 'tag' in opt_dict:
            self.figure_tag.setEditText(str(opt_dict['tag'])[1:-1])
        else:
            self.figure_tag.setEditText("")
        
        if '_args' in opt_dict:
            self.graphicx_edit.setText(opt_dict['_args'])

        # build locator
        if 'filename' in opt_dict:
            # set using basedir of tex file
            fname = opt_dict['filename']
            if not os.path.isabs(fname):
                source_dir = os.path.dirname(str(self.source_edit.text()))
                #print 'source_dir:', str(self.source_edit.text()), source_dir
                fname = os.path.join(source_dir, fname)
            
            locator = FileLocator(fname)
        elif 'host' in opt_dict:
            if 'port' not in opt_dict:
                opt_dict['port'] = '3306'
            port = int(opt_dict['port'])
            locator = DBLocator(opt_dict['host'], port,
                                opt_dict['db'], '', '', 
                                obj_id=opt_dict['vtid'])
        else:
            locator = None
        if locator is not None:
            self.figure_ref.setText(locator.to_url())
        else:
            self.figure_ref.setText("")
        self.figure_ref.locator = locator

        self.readImage(opt_dict)

    def getFigureInfo(self):
        # return an opt_dict here...
        opt_dict = self.doCheckLink(False)
        if self.figure_type.currentIndex() == 1:
            opt_dict['showworkflow'] = None
        elif self.figure_type.currentIndex() == 2:
            opt_dict['showtree'] = None

        version_text = str(self.figure_version.text())
        if version_text:
            opt_dict['version'] = version_text
        tag_text = str(self.figure_tag.currentText())
        if self.figure_smart.isChecked() and tag_text:
            opt_dict['tag'] = '{%s}' % tag_text

        graphicx_text = str(self.graphicx_edit.text())
        if graphicx_text:
            opt_dict['_args'] = graphicx_text

        locator = self.figure_ref.locator
        if isinstance(locator, DBLocator):
            opt_dict['host'] = locator.host
            opt_dict['port'] = locator.port
            opt_dict['db'] = locator.db
            opt_dict['vtid'] = locator.obj_id
        else:
            fname = str(self.source_edit.text())
            if os.path.dirname(fname) == os.path.dirname(locator.name):
                # do relative
                # FIXME do relative via a checkbox!
                opt_dict['filename'] = os.path.basename(locator.name)
            else:
                opt_dict['filename'] = locator.name

        return opt_dict

    def figureSelected(self):
        selected_items = self.figure_list.selectedItems()
        if self.selected_item is not None:
            self.selected_item.update_opt_dict(self.getFigureInfo())
            #print "NEW OPTIONS:", self.selected_item.opt_dict
        if len(selected_items) < 1:
            self.gbEmbedOpt.setEnabled(False)
            self.gbDownOpt.setEnabled(False)
            self.def_group.setEnabled(False)
            self.selected_item = None
        else:
            self.gbEmbedOpt.setEnabled(True)
            self.gbDownOpt.setEnabled(True)
            self.def_group.setEnabled(True)
            item = selected_items[0]
            #print "OPTIONS:", item.opt_dict
            self.setFigureInfo(item.opt_dict)
            self.selected_item = item

    def addFigure(self):
        item = QLatexFigureItem({})
        item.setText("Figure %d" % (self.figure_list.count() + 1))
        self.figure_list.addItem(item)

    def deleteFigure(self):
        selected_items = self.figure_list.selectedItems()
        if len(selected_items) < 1:
            return
        idx = self.figure_list.row(selected_items[0])
        self.figure_list.takeItem(idx)

    def saveLatex(self):
        fname = '/vistrails/src/git/extensions/latex/head2.tex'
        f = open(fname, 'w')
        raw_texts = self.texts[0]
        figure_items = [self.figure_list.item(i) 
                        for i in xrange(self.figure_list.count())]
        for i in xrange(len(raw_texts)):
            f.write(raw_texts[i])
            if i < len(figure_items):
                f.write(build_vt_command(figure_items[i].get_opt_dict()))
        f.close()

    def openVersion(self):
        pass

    def useCurrent(self):
        pass

class QVersionEmbed(QtGui.QWidget, QVistrailsPaletteInterface):
    def __init__(self, parent=None, f=QtCore.Qt.WindowFlags()):
        QtGui.QWidget.__init__(self, parent, 
                               f | QtCore.Qt.Tool | QtCore.Qt.WindowStaysOnTopHint)
        self.setWindowTitle('Publish Workflow')
        self.setFocusPolicy(QtCore.Qt.ClickFocus)
        self.versionNumber = None
        self.versionTag = ''
        label1 = QtGui.QLabel("Embed:")
        self.cbcontent = QtGui.QComboBox()
        self.cbcontent.setEditable(False)
        items = ["Workflow Results", "Workflow Graph", "History Tree Graph"]
        self.cbcontent.addItems(items)
        label2 = QtGui.QLabel("In:")
        
        self.cbtype = QtGui.QComboBox()
        self.cbtype.setEditable(False)
        items = ["Wiki", "Latex", "Shared Memory"]
        self.cbtype.addItems(items)
        
        self.controller = None
        self.pptag = 'Image(s) from (%s,%s,%s,%s,%s)'
        
        #options
        self.gbEmbedOpt = QtGui.QGroupBox("Embed Options")
        self.chbPdf = QtGui.QCheckBox("As PDF")
        self.chbSmartTag = QtGui.QCheckBox("Smart Tag")
        self.chbCache = QtGui.QCheckBox("Cache Images")
        self.chbLatexVTL = QtGui.QCheckBox("Include .vtl")
        self.chbLatexVTL.setEnabled(False)
        
        gblayout = QtGui.QGridLayout()
        gblayout.addWidget(self.chbPdf, 0, 0)
        gblayout.addWidget(self.chbSmartTag, 0, 1)
        gblayout.addWidget(self.chbCache, 1, 0)
        gblayout.addWidget(self.chbLatexVTL, 1, 1)
        self.gbEmbedOpt.setLayout(gblayout)
        
        self.gbDownOpt = QtGui.QGroupBox("Download Options")
        self.chbWorkflow = QtGui.QCheckBox("Include Workflow")
        self.chbFullTree = QtGui.QCheckBox("Include Full Tree")
        self.chbFullTree.setEnabled(False)
        self.chbExecute = QtGui.QCheckBox("Execute Workflow")
        self.chbSpreadsheet = QtGui.QCheckBox("Show Spreadsheet Only")
        
        gblayout = QtGui.QGridLayout()
        gblayout.addWidget(self.chbWorkflow, 0, 0)
        gblayout.addWidget(self.chbFullTree, 0, 1)
        gblayout.addWidget(self.chbExecute, 1, 0)
        gblayout.addWidget(self.chbSpreadsheet, 1, 1)
        self.gbDownOpt.setLayout(gblayout)
        
        self.embededt = QtGui.QTextEdit(self)
        self.embededt.setAcceptRichText(False)
        self.embededt.setReadOnly(False)
        # self.exportHtml = '<a href="export">Export...</a>'
        # self.copyHtml = '<a href="copy">Copy to Clipboard</a>'
        # self.copylabel = QtGui.QLabel(self.copyHtml)
        # self.copylabel.setCursor(QtCore.Qt.PointingHandCursor)
        self.copyButton = QtGui.QPushButton() # "Copy to Clipboard")
        self.copyButton.setMinimumSize(50, 30)
        self.link = "copy"
        
        self.helpLabel = QtGui.QLabel()
        # "After making your selection, "
        #                               "click on 'Copy To Clipboard'. "
        #                               "The code changes based on your "
        #                               "selection.")
        self.helpLabel.setWordWrap(True)
        
        font = QtGui.QFont("Arial", 10, QtGui.QFont.Normal)
        font.setItalic(True)
        self.helpLabel.setFont(font)
        
        layout = QtGui.QGridLayout()
        layout.addWidget(label1,0,0)
        layout.addWidget(self.cbcontent,0,1)
        layout.addWidget(label2,1,0)
        layout.addWidget(self.cbtype,1,1)
        layout.addWidget(self.gbEmbedOpt,2,0,1,2)
        layout.addWidget(self.gbDownOpt,3,0,1,2)
        layout.addWidget(self.embededt,4,0,1,-1)
        layout.addWidget(self.helpLabel,5,0,1,2)
        layout.addWidget(self.copyButton,6,1,QtCore.Qt.AlignRight)

        self.setLayout(layout)
        self.changeEmbedType("Wiki")
        
        #connect signals
        self.connect(self.cbtype,
                     QtCore.SIGNAL("activated(const QString &)"),
                     self.changeEmbedType)
        
        # self.connect(self.copylabel,
        #              QtCore.SIGNAL("linkActivated(const QString &)"),
        #              self.linkActivated)

        self.connect(self.copyButton,
                     QtCore.SIGNAL("clicked()"),
                     self.copyClicked)
        
        self.connect(self.cbcontent,
                     QtCore.SIGNAL("activated(const QString &)"),
                     self.changeOption)
        
        optlist = [self.cbcontent,
                   self.chbPdf,
                   self.chbSmartTag,
                   self.chbCache,
                   self.chbLatexVTL,
                   self.chbWorkflow,
                   self.chbFullTree,
                   self.chbExecute,
                   self.chbSpreadsheet]
        for cb in optlist:
            self.connect(cb, QtCore.SIGNAL("toggled(bool)"),
                         self.changeOption)
        #special cases
        self.connect(self.chbWorkflow, QtCore.SIGNAL("toggled(bool)"),
                     self.changeIncludeWorkflow)
        self.connect(self.chbSpreadsheet, QtCore.SIGNAL("toggled(bool)"),
                     self.changeShowSpreadsheet)
        self.connect(self.chbExecute, QtCore.SIGNAL("toggled(bool)"),
                     self.changeExecute)
        self.connect(self.cbcontent,
                     QtCore.SIGNAL("activated(const QString &)"),
                     self.changeContent)
        self.connect(self.chbSmartTag, QtCore.SIGNAL("toggled(bool)"),
                     self.changeSmartTag)
        self.connect(self.chbCache, QtCore.SIGNAL("toggled(bool)"),
                     self.changeCache)
        
    def set_controller(self, controller):
        self.controller = controller
        
    def closeEvent(self,e):
        """ closeEvent(e: QCloseEvent) -> None
        Doesn't allow the Legend widget to close, but just hide
        instead
        
        """
        e.ignore()
        self.hide()
    
    def focusInEvent(self, event):
        if self.controller:
            if self.controller.locator and \
               not isinstance(self.controller.locator, UntitledLocator):
                from vistrails.gui.vistrails_window import _app
                _app.ensureVistrail(self.controller.locator)
                    
                    
    def checkLocator(self):
        """checkLocator() -> bool
        Only vistrails on a database are allowed to embed a tag"""
        result = False
        if self.controller:
            if self.controller.locator and \
               not isinstance(self.controller.locator, UntitledLocator):
                title = "Embed Options for %s"%self.controller.locator.name
                self.setWindowTitle(title)
                result = True
        return result

    def checkControllerStatus(self):
        """checkControllerStatus() -> bool
        this checks if the controller has saved the latest changes """
        result = False
        if self.controller:
            result = not self.controller.changed
        return result
    
    def updateEmbedText(self):
        ok = (self.checkLocator() and self.checkControllerStatus() and
              self.versionNumber > 0)
        self.embededt.setEnabled(ok)
        self.copyButton.setEnabled(ok)
        self.embededt.setText('')

        if self.controller and self.versionNumber > 0:
            if self.controller.locator and \
               not isinstance(self.controller.locator, UntitledLocator) and \
               not self.controller.changed:
                loc = self.controller.locator
                if hasattr(loc,'host'):
                    self.updateCbtype('db')    
                elif hasattr(loc, 'name'):
                    self.updateCbtype('file')
                        
                if self.versionTag != "":
                    self.chbSmartTag.setEnabled(True)
                else:
                    self.chbSmartTag.setChecked(False)
                    self.chbSmartTag.setEnabled(False)
                    
                self.setEmbedText()
            elif self.controller.changed:
                self.embededt.setPlainText('You must save your vistrail to proceed')
            else:
                self.embededt.setPlainText('')
                
    def setEmbedText(self):
        #check options
        options = {}
        options['content'] = str(self.cbcontent.currentText())
        options['pdf'] = self.chbPdf.isChecked()
        options['smartTag'] = self.chbSmartTag.isChecked()
        options['buildalways'] = not self.chbCache.isChecked()
        options['includeWorkflow'] = self.chbWorkflow.isChecked()
        options['includeFullTree'] = self.chbFullTree.isChecked()
        options['execute'] = self.chbExecute.isChecked()
        options['showspreadsheetonly'] = self.chbSpreadsheet.isChecked()
        
        if str(self.cbtype.currentText()) == "Wiki":
            text = self.buildWikiTag(options)
        elif str(self.cbtype.currentText()) == "Latex":
            options['getvtl'] = self.chbLatexVTL.isChecked()
            text = self.buildLatexCommand(options)
        elif str(self.cbtype.currentText()) == "Shared Memory":
            text = self.pptag
        self.embededt.setPlainText(text)
            
    def updateVersion(self, versionNumber):
        self.versionNumber = versionNumber
        if versionNumber > 0:
            self.versionTag = self.controller.vistrail.getVersionName(self.versionNumber)
        self.updateEmbedText()

    def copyClicked(self):
        if self.link == 'copy':
            clipboard = QtGui.QApplication.clipboard()
            clipboard.setText(self.embededt.toPlainText())
        elif self.link=='export':
            app = QtCore.QCoreApplication.instance()
            app.builderWindow.interactiveExportCurrentPipeline()

    # def linkActivated(self, link):
    #     if link=='copy':
    #         clipboard = QtGui.QApplication.clipboard()
    #         clipboard.setText(self.embededt.toPlainText())
    #     elif link=='export':
    #         app = QtCore.QCoreApplication.instance()
    #         app.builderWindow.interactiveExportCurrentPipeline()
    
    def switchType(self, text):
        if text == 'Wiki':
            index = 0
        elif text == "Latex":
            if self.cbtype.count() > 1:
                index = 1
            else:
                index = 0
        else:
            index = 2
        self.cbtype.setCurrentIndex(index)
        self.changeEmbedType(text)
        
    def changeEmbedType(self, text):
        """changeEmbedType(text) -> None
        This is called when the combobox is activated so the proper gui elements
        are enabled.
        
        """
        if text!='Shared Memory':
            # self.copylabel.setText(self.copyHtml)
            self.link = 'copy'
            self.copyButton.setText("Copy to Clipboard")
            self.helpLabel.setText("After making your selections, "
                                   "click on 'Copy To Clipboard'. "
                                   "The code changes based on your "
                                   "selections.")
        else:
            # self.copylabel.setText(self.exportHtml)
            self.link = 'export'
            self.copyButton.setText("Export...")
            self.helpLabel.setText("After making your selections, "
                                   "click 'Export...'. The code changes "
                                   "based on your selections.")
        self.chbLatexVTL.setEnabled(text == 'Latex')
        self.chbPdf.setEnabled(text == 'Latex')
        if self.controller is not None:
            self.updateEmbedText()
        
    def changeOption(self, value):
        self.updateEmbedText()
        
    def changeContent(self, text):
        if text == "Workflow Results":
            self.chbExecute.setEnabled(True)
            self.chbSpreadsheet.setEnabled(True)
            self.chbCache.setEnabled(True)
            self.chbSmartTag.setEnabled(True)
        else:
            self.chbExecute.setChecked(False)
            self.chbSpreadsheet.setChecked(False)
            self.chbExecute.setEnabled(False)
            self.chbSpreadsheet.setEnabled(False)
            if text == "History Tree Graph":
                self.chbCache.setChecked(False)
                self.chbCache.setEnabled(False)
                self.chbSmartTag.setChecked(False)
                self.chbSmartTag.setEnabled(False)
            else:
                self.chbCache.setEnabled(True)
                self.chbSmartTag.setEnabled(True)
                
    def updateCbtype(self, type):
        currentText = self.cbtype.currentText()
        self.cbtype.clear()
        items = []
        if type == 'db':
            items = ["Wiki", "Latex", "Shared Memory"]
        elif type == 'file':
            items = ["Latex"]
        self.cbtype.addItems(items)
        try:
            index = items.index(currentText)
        except ValueError:
            index = -1
        if index > 0:
            self.cbtype.setCurrentIndex(index)
        text = str(self.cbtype.currentText())
        self.chbLatexVTL.setVisible(text == 'Latex')
        self.chbPdf.setEnabled(text == 'Latex')
            
    def buildLatexCommand(self, options):
        text = '\\vistrail['
        loc = self.controller.locator
        if hasattr(loc, 'host'):
            text += 'host=%s,\ndb=%s,\nport=%s,\nvtid=%s,\n'% (loc.host,
                                                               loc.db,
                                                               loc.port,
                                                               loc.obj_id)
        else:
            text += 'filename=%s,\n' % os.path.basename(loc.name)
        if options['content'] != "History Tree Graph":    
            text += 'version=%s,\n'%self.versionNumber
            if options['smartTag']:
                text += 'tag={%s},\n'%self.versionTag
        if options['pdf']:
            text += 'pdf,\n'
        if options['buildalways']:
            text+= 'buildalways,\n'
        if options['getvtl']:
            text += 'getvtl,\n'
        if options['includeWorkflow']:
            text+= 'embedworkflow,\n'
            if options['includeFullTree']:
                text += 'includefulltree,\n'
        if options['content'] == "Workflow Graph":
            text += 'showworkflow,\n'
        elif options['content'] == "History Tree Graph":
            text += 'showtree,\n'
        else:
            if options['execute']:
                text += 'execute,\n'
            if options['showspreadsheetonly']:
                text += 'showspreadsheetonly,\n'
        
        text = text[0:-2] + "]{}"
        return text        

    def buildWikiTag(self, options):
        text = '<vistrail '
        loc = self.controller.locator
        
        text += 'host="%s" db="%s" port="%s" vtid="%s" '% (loc.host,
                                                            loc.db,
                                                            loc.port,
                                                            loc.obj_id)
        if options['content'] != "History Tree Graph":
            text += 'version="%s" '%self.versionNumber
            if options['smartTag']:
                text += 'tag="%s " '%self.versionTag
        if options['buildalways']:
            text+= 'buildalways="True" '
        if options['includeWorkflow']:
            text+= 'embedworkflow="True" '
            if options['includeFullTree']:
                text += 'includefulltree="True" '
        if options['content'] == "Workflow Graph":
            text += 'showworkflow="True" ' #"Workflow Results" << "Workflow Graph" << "History Tree Graph";
        elif options['content'] == "History Tree Graph":
            text += 'showtree="True" '
        else:
            if options['execute']:
                text += 'execute="True" '
            if options['showspreadsheetonly']:
                text += 'showspreadsheetonly="True" '
        
        text += "/>"
        return text        

    def changeIncludeWorkflow(self,checked):
        self.chbFullTree.setEnabled(checked)
        if self.cbcontent.currentText() == "History Tree Graph":
            self.chbFullTree.setChecked(checked)
        
    def changeShowSpreadsheet(self, checked):
        if checked:
            self.chbExecute.setChecked(True)
            
    def changeExecute(self, checked):
        if not checked:
            self.chbSpreadsheet.setChecked(False)
            
    def changeSmartTag(self, checked):
        if checked and self.cbtype.currentText() == 'Latex':
            self.chbCache.setChecked(False)
            
    def changeCache(self, checked):
        if checked and self.cbtype.currentText() == 'Latex':
            self.chbSmartTag.setChecked(False)



#!/usr/bin/env python

# This example demonstrates how to use boolean combinations of implicit
# functions to create a model of an ice cream cone.

import vtk
from vtk.util.colors import chocolate, mint

# Create implicit function primitives. These have been carefully
# placed to give the effect that we want. We are going to use various
# combinations of these functions to create the shape we want; for
# example, we use planes intersected with a cone (which is infinite in
# extent) to get a finite cone.
cone = vtk.vtkCone()
cone.SetAngle(20)
vertPlane = vtk.vtkPlane()
vertPlane.SetOrigin(.1, 0, 0)
vertPlane.SetNormal(-1, 0, 0)
basePlane = vtk.vtkPlane()
basePlane.SetOrigin(1.2, 0, 0)
basePlane.SetNormal(1, 0, 0)
iceCream = vtk.vtkSphere()
iceCream.SetCenter(1.333, 0, 0)
iceCream.SetRadius(0.5)
bite = vtk.vtkSphere()
bite.SetCenter(1.5, 0, 0.5)
bite.SetRadius(0.25)

# Combine primitives to build ice-cream cone. Clip the cone with planes.
theCone = vtk.vtkImplicitBoolean()
theCone.SetOperationTypeToIntersection()
theCone.AddFunction(cone)
theCone.AddFunction(vertPlane)
theCone.AddFunction(basePlane)

# Take a bite out of the ice cream.
theCream = vtk.vtkImplicitBoolean()
theCream.SetOperationTypeToDifference()
theCream.AddFunction(iceCream)
theCream.AddFunction(bite)

# The sample function generates a distance function from the implicit
# function (which in this case is the cone). This is then contoured to
# get a polygonal surface.
theConeSample = vtk.vtkSampleFunction()
theConeSample.SetImplicitFunction(theCone)
theConeSample.SetModelBounds(-1, 1.5, -1.25, 1.25, -1.25, 1.25)
theConeSample.SetSampleDimensions(60, 60, 60)
theConeSample.ComputeNormalsOff()
theConeSurface = vtk.vtkContourFilter()
theConeSurface.SetInputConnection(theConeSample.GetOutputPort())
theConeSurface.SetValue(0, 0.0)
coneMapper = vtk.vtkPolyDataMapper()
coneMapper.SetInputConnection(theConeSurface.GetOutputPort())
coneMapper.ScalarVisibilityOff()
coneActor = vtk.vtkActor()
coneActor.SetMapper(coneMapper)
coneActor.GetProperty().SetColor(chocolate)

# The same here for the ice cream.
theCreamSample = vtk.vtkSampleFunction()
theCreamSample.SetImplicitFunction(theCream)
theCreamSample.SetModelBounds(0, 2.5, -1.25, 1.25, -1.25, 1.25)
theCreamSample.SetSampleDimensions(60, 60, 60)
theCreamSample.ComputeNormalsOff()
theCreamSurface = vtk.vtkContourFilter()
theCreamSurface.SetInputConnection(theCreamSample.GetOutputPort())
theCreamSurface.SetValue(0, 0.0)
creamMapper = vtk.vtkPolyDataMapper()
creamMapper.SetInputConnection(theCreamSurface.GetOutputPort())
creamMapper.ScalarVisibilityOff()
creamActor = vtk.vtkActor()
creamActor.SetMapper(creamMapper)
creamActor.GetProperty().SetColor(mint)

# Create the usual rendering stuff
ren = vtk.vtkRenderer()
renWin = vtk.vtkRenderWindow()
renWin.AddRenderer(ren)
iren = vtk.vtkRenderWindowInteractor()
iren.SetRenderWindow(renWin)

# Add the actors to the renderer, set the background and size
ren.AddActor(coneActor)
ren.AddActor(creamActor)
ren.SetBackground(1, 1, 1)
renWin.SetSize(250, 250)
ren.ResetCamera()
ren.GetActiveCamera().Roll(90)
ren.GetActiveCamera().Dolly(1.5)
ren.ResetCameraClippingRange()

iren.Initialize()
renWin.Render()
iren.Start()
